@@ -1,7 +1,15 @@
 import gtk, gwui, microblog, resources, util, json
 from microblog.util.const import *
+# Try to import * from custom, install custom.py to include packaging 
+# customizations like distro API keys, etc
+try:
+  from microblog.util.custom import *
+except:
+  pass
+
 import gwibber.microblog.util
 import mx.DateTime
+import re
 
 from gettext import lgettext as _
 
@@ -15,7 +23,7 @@ class MessageAction:
     
   @classmethod
   def include(self, client, msg):
-    return self.__name__ in client.model.services[msg["protocol"]]["features"]
+    return self.__name__ in client.model.services[msg["service"]]["features"]
 
   @classmethod
   def action(self, w, client, msg):
@@ -42,13 +50,17 @@ class thread(MessageAction):
 class retweet(MessageAction):
   icon = "mail-forward"
   label = _("R_etweet")
+  PARSE_GROUP = re.compile("\B!([A-Za-z0-9_\-]+|@[A-Za-z0-9_\-]$)")
 
   @classmethod
   def action(self, w, client, msg):
-    if "retweet" in client.model.services[msg["protocol"]]["features"]:
+    if "retweet" in client.model.services[msg["service"]]["features"]:
       text = RETWEET_FORMATS[client.model.settings["retweet_style"]]
-      symbol = "RD" if msg["protocol"] == "identica" else "RT"
+      symbol = "RD" if msg["service"] == "identica" else "RT"
       text = text.format(text=msg["text"], nick=msg["sender"]["nick"], R=symbol)
+      #replace ! with # if identica service
+      if msg["service"] == "identica" and client.model.settings["global_retweet"]:
+          text = self.PARSE_GROUP.sub('#\\1', text)
 
       if not client.model.settings["global_retweet"]:
         client.message_target.set_target(msg, "repost")
@@ -58,6 +70,14 @@ class retweet(MessageAction):
       buf = client.input.textview.get_buffer()
       buf.place_cursor(buf.get_end_iter())
 
+class private(MessageAction):
+  icon = "mail-reply-sender"
+  label = _("_Direct Message")
+
+  @classmethod
+  def action(self, w, client, msg):
+    client.private(msg)
+
 class like(MessageAction):
   icon = "bookmark_add"
   label = _("_Like this message")
@@ -74,6 +94,12 @@ class like(MessageAction):
     image = resources.get_ui_asset("gwibber.svg")
     expire_timeout = 5000
     n = gwibber.microblog.util.notify(_("Liked"), _("You have marked this message as liked."), image, expire_timeout)
+  
+  @classmethod
+  def include(self, client, msg):
+    if "like" in client.model.services[msg["service"]]["features"]:
+      if not msg.get("sender", {}).get("is_me", 0):
+        return True
 
 class delete(MessageAction):
   icon = "gtk-delete"
@@ -94,7 +120,7 @@ class delete(MessageAction):
 
   @classmethod
   def include(self, client, msg):
-    if "delete" in client.model.services[msg["protocol"]]["features"]:
+    if "delete" in client.model.services[msg["service"]]["features"]:
       if msg.get("sender", {}).get("is_me", 0):
         return True
 
@@ -121,6 +147,17 @@ class read(MessageAction):
   def include(self, client, msg):
     return "url" in msg
 
+class list(MessageAction):
+  @classmethod
+  def action(self, w, client, acct=None, user=None, id=None, name=None):
+    if acct and user and id and name:
+      client.add_stream({
+        "name": name,
+        "account": acct,
+        "operation": "list",
+        "parameters": {"user": user, "id": id},
+      })
+
 class user(MessageAction):
   icon = "face-monkey"
   label = _("View user _Profile")
@@ -137,7 +174,7 @@ class user(MessageAction):
 class menu(MessageAction):
   @classmethod
   def action(self, w, client, msg):
-    client.on_message_action_menu(msg)
+    client.on_message_action_menu(msg, w)
 
 class tag(MessageAction):
   @classmethod
@@ -145,7 +182,23 @@ class tag(MessageAction):
     client.add_stream({
       "name": "#%s" % query,
       "query": "#%s" % query,
-    }, COUCH_TYPE_SEARCH)
+    })
+
+class translate(MessageAction):
+  icon = "config-language"
+  label = _("Tra_nslate")
+
+  @classmethod
+  def action(self, w, client, msg):
+    ## Need to figure out the user's language instead of default to EN
+    script = """document.getElementById("text-%s").innerHTML = %s"""
+    text = gwibber.microblog.util.getbus("Translate").Translate(msg["text"], "", "en")
+    if isinstance(w.messages.message_view, gwui.WebUi):
+      w.messages.message_view.execute_script(script % (msg["id"], json.dumps(text)))
+
+  @classmethod
+  def include(self, client, msg):
+    return True
 
 class group(MessageAction):
   icon = "face-monkey"
@@ -162,8 +215,8 @@ class tomboy(MessageAction):
   @classmethod
   def action(self, w, client, msg):
     util.create_tomboy_note(
-      _("%(protocol_name)s message from %(sender)s at %(time)s\n\n%(message)s\n\nSource: %(url)s") % {
-        "protocol_name": client.model.services[msg["protocol"]]["name"],
+      _("%(service_name)s message from %(sender)s at %(time)s\n\n%(message)s\n\nSource: %(url)s") % {
+        "service_name": client.model.services[msg["service"]]["name"],
         "sender": msg["sender"]["name"],
         "time": mx.DateTime.DateTimeFromTicks(msg["time"]).localtime().strftime(),
         "message": msg["text"],
@@ -174,4 +227,4 @@ class tomboy(MessageAction):
   def include(self, client, msg):
     return gwibber.microblog.util.service_is_running("org.gnome.Tomboy")
 
-MENU_ITEMS = [reply, retweet, read, user, like, delete, tomboy]
+MENU_ITEMS = [reply, retweet, private, read, user, like, delete, tomboy, translate]
