@@ -1,9 +1,9 @@
 #!/usr/bin/env python
 
 import gtk, gobject, gwui, util, resources, actions, json, gconf
+import subprocess, os, sqlite3, time, datetime
+from microblog import config
 import microblog.util
-import subprocess, os
-import time, datetime
 
 import gettext
 from gettext import lgettext as _
@@ -11,11 +11,15 @@ if hasattr(gettext, 'bind_textdomain_codeset'):
     gettext.bind_textdomain_codeset('gwibber','UTF-8')
 gettext.textdomain('gwibber')
 
-from microblog.util.couch import RecordMonitor
-from desktopcouch.records.server import CouchDatabase
-from desktopcouch.records.record import Record as CouchRecord
 from gwibber.microblog.util import log
 from microblog.util.const import *
+# Try to import * from custom, install custom.py to include packaging 
+# customizations like distro API keys, etc
+try:
+  from microblog.util.custom import *
+except:
+  pass
+
 from dbus.mainloop.glib import DBusGMainLoop
 import dbus, dbus.service
 
@@ -32,39 +36,89 @@ class GwibberClient(gtk.Window):
     self.service = self.model.daemon
     self.service.connect_to_signal("LoadingStarted", self.on_loading_started)
     self.service.connect_to_signal("LoadingComplete", self.on_loading_complete)
+    self.service.connect_to_signal("IndicatorInterestAdded", self.on_indicator_interest_added)
+    self.service.connect_to_signal("IndicatorInterestRemoved", self.on_indicator_interest_removed)
 
     self.connection = microblog.util.getbus("Connection")
     self.connection.connect_to_signal("ConnectionOnline", self.on_connection_online)
     self.connection.connect_to_signal("ConnectionOffline", self.on_connection_offline)
 
-    microblog.util.getbus("Streams").connect_to_signal("SettingChanged", self.on_setting_changed)
+    self.accounts = microblog.util.getbus("Accounts")
 
-    if len(json.loads(self.service.GetAccounts())) == 0:
-      import upgrade
-      account_migrate = upgrade.GwibberAccountMigrate()
-      if not account_migrate.run():
-        if os.path.exists(os.path.join("bin", "gwibber-accounts")):
-          cmd = os.path.join("bin", "gwibber-accounts")
-        else:
-          cmd = "gwibber-accounts"
-        ret = 1
-        while ret != 0:
-          ret = subprocess.call([cmd])
-      self.service.Refresh()
-
-    self.gc = gconf.client_get_default()
-    self.gc.add_dir("/desktop/gnome/interface/font_name", gconf.CLIENT_PRELOAD_NONE)
-    self.gc.notify_add("/desktop/gnome/interface/font_name", self.on_font_changed)
+    first_run = False
+    # check for existing accounts
+    if len(json.loads(self.accounts.List())) == 0:
+      # if there are no accounts configured, prompt the user to add some
+      first_run = True
+      if os.path.exists(os.path.join("bin", "gwibber-accounts")):
+        cmd = os.path.join("bin", "gwibber-accounts")
+      else:
+        cmd = "gwibber-accounts"
+      ret = 1
+      while ret != 0:
+        ret = subprocess.call([cmd])
 
     self.setup_ui()
 
+    if first_run:
+      # Since we didn't have accounts configured when gwibber-service started
+      # force it to refresh now that accounts have been added
+      self.service.Refresh()
+
+    # Migrate the autostart gconf key to the new location
+    if config.GCONF.get("/apps/gwibber/autostart"):
+      config.GCONF.set_bool("/apps/gwibber/preferences/autostart", True)
+      config.GCONF.unset('/apps/gwibber/autostart')
+
+    config.GCONF.add_dir("/desktop/gnome/interface/document_font_name", gconf.CLIENT_PRELOAD_NONE)
+    config.GCONF.notify_add("/desktop/gnome/interface/document_font_name", self.on_font_changed)
+    config.GCONF.add_dir("/desktop/gnome/font_rendering", gconf.CLIENT_PRELOAD_NONE)
+    config.GCONF.notify_add("/desktop/gnome/font_rendering", self.on_font_changed)
+    config.GCONF.add_dir("/apps/gwibber/preferences", gconf.CLIENT_PRELOAD_NONE)
+
+    for x in ["hide_taskbar_entry", "show_tray_icon"]:
+      self.model.settings.notify(x, self.on_setting_changed)
+
+    # Only use the notification area icon if there is no interest from the 
+    # messaging menu indicator
+    if self.service.IndicatorInterestCheck():
+      self.tray_icon.set_visible(False)
+    else:
+      self.tray_icon.set_visible(self.model.settings["show_tray_icon"])
+
+    self.set_property("skip-taskbar-hint", self.model.settings["hide_taskbar_entry"])
+
     # set state online/offline
     if not self.connection.isConnected():
       log.logger.info("Setting to Offline")
       self.actions.get_action("refresh").set_sensitive(False)
 
-  def on_setting_changed(self):
-    self.update_view()
+    # Delay resizing input area or else it doesn't work
+    gobject.idle_add(lambda: self.input_splitter.set_position(int(self.model.settings["window_splitter"])))
+
+    # Delay resizing the sidebar area or else it doesn't work
+    gobject.idle_add(lambda: hasattr(self.stream_view, "splitter") and self.stream_view.splitter.set_position(self.model.settings["sidebar_splitter"]))
+
+    # Apply the user's settings 
+    gobject.idle_add(lambda: self.resize(*self.model.settings["window_size"]))
+    gobject.idle_add(lambda: self.move(*self.model.settings["window_position"]))
+
+  def on_indicator_interest_added(self):
+    # Hide the notification area icon if there is interest from the 
+    # messaging menu indicator
+    self.tray_icon.set_visible(False)
+
+  def on_indicator_interest_removed(self):
+    # Show the notification area icon if there is no interest from the 
+    # messaging menu indicator
+    self.tray_icon.set_visible(True)
+
+  def on_setting_changed(self, gc, x, pref, y):
+    if "show_tray_icon" in pref.key:
+      self.tray_icon.set_visible(pref.value.get_bool())
+
+    if "hide_taskbar_entry" in pref.key:
+      self.set_property("skip-taskbar-hint", pref.value.get_bool())
 
   def on_font_changed(self, *args):
     self.update_view()
@@ -78,6 +132,14 @@ class GwibberClient(gtk.Window):
     menu_ui = self.setup_menu()
     self.add_accel_group(menu_ui.get_accel_group())
 
+    def on_tray_menu_popup(i, b, a):
+      menu_ui.get_widget("/menu_tray").popup(None, None,
+          gtk.status_icon_position_menu, b, a, self.tray_icon)
+
+    self.tray_icon = gtk.status_icon_new_from_file(resources.get_ui_asset("gwibber.svg"))
+    self.tray_icon.connect("activate", self.on_toggle_window_visibility)
+    self.tray_icon.connect("popup-menu", on_tray_menu_popup)
+
     # Create animated loading spinner
     self.loading_spinner = gtk.Image()
     menu_spin = gtk.ImageMenuItem("")
@@ -92,9 +154,12 @@ class GwibberClient(gtk.Window):
     menubar = menu_ui.get_widget("/menubar_main")
     menubar.append(menu_spin)
 
-    view_class = getattr(gwui,
-        "MultiStreamUi" if len(self.model.settings["streams"]) > 1
-        else "SingleStreamUi")
+    # Load the user's saved streams
+    streams = json.loads(self.model.settings["streams"])
+    streams = [dict((str(k), v) for k, v in s.items()) for s in streams]
+    
+    # Use the multicolumn mode if there are multiple saved streams
+    view_class = getattr(gwui, "MultiStreamUi" if len(streams) > 1 else "SingleStreamUi")
 
     self.stream_view = view_class(self.model)
     self.stream_view.connect("action", self.on_action)
@@ -119,7 +184,6 @@ class GwibberClient(gtk.Window):
 
     self.message_target = gwui.AccountTargetBar(self.model)
     self.message_target.pack_end(self.button_send, False)
-    #self.message_target.pack_end(self.character_count, False)
     self.message_target.connect("canceled", self.on_reply_cancel)
 
     content = gtk.VBox(spacing=5)
@@ -133,18 +197,28 @@ class GwibberClient(gtk.Window):
 
     self.add(layout)
 
-    self.window_size = self.gc.get_list(GCONF_CLIENT_DIR + "window_size", gconf.VALUE_INT) or LOCAL_SETTINGS["window_size"]
-    self.window_position = self.gc.get_list(GCONF_CLIENT_DIR + "window_position", gconf.VALUE_INT) or LOCAL_SETTINGS["window_position"]
-    self.window_splitter = self.gc.get_int(GCONF_CLIENT_DIR + "window_splitter") or LOCAL_SETTINGS["window_splitter"]
     # Apply the user's settings 
-    self.resize(self.window_size[0], self.window_size[1])
-    self.move(self.window_position[0], self.window_position[1])
-    self.input_splitter.set_position(self.window_splitter)
+    self.resize(*self.model.settings["window_size"])
+    self.move(*self.model.settings["window_position"])
+    self.input_splitter.set_position(self.model.settings["window_splitter"])
+    
+    if hasattr(self.stream_view, "splitter"):
+      self.stream_view.splitter.set_position(
+          self.model.settings["sidebar_splitter"])
+    
     self.show_all()
 
-    self.stream_view.set_state(self.model.settings["streams"] or DEFAULT_SETTINGS["streams"])
+    self.stream_view.set_state(streams)
     self.update_menu_availability()
 
+  def on_toggle_window_visibility(self, w):
+    if self.get_property("visible"):
+      self.last_position = self.get_position()
+      self.hide()
+    else:
+      self.present()
+      self.move(*self.last_position)
+
   def set_view(self, view=None):
     state = None
     if view: self.view_class = getattr(gwui, view)
@@ -211,7 +285,7 @@ class GwibberClient(gtk.Window):
       ("menu_edit", None, _("_Edit")),
       ("menu_help", None, _("_Help")),
 
-      ("refresh", gtk.STOCK_REFRESH, _("_Refresh"), "<ctrl>R", None, self.on_refresh),
+      ("refresh", gtk.STOCK_REFRESH, _("_Refresh"), "F5", None, self.on_refresh),
       ("search", gtk.STOCK_FIND, _("_Search"), "<ctrl>F", None, self.on_search),
       ("accounts", None, _("_Accounts"), "<ctrl><shift>A", None, self.on_accounts),
       ("preferences", gtk.STOCK_PREFERENCES, _("_Preferences"), "<ctrl>P", None, self.on_preferences),
@@ -227,10 +301,15 @@ class GwibberClient(gtk.Window):
       ("help_report", None, _("Report A Problem..."), None, None, lambda *a: util.load_url(BUG_URL)),
     ])
 
-
     ui = gtk.UIManager()
     ui.insert_action_group(self.actions, pos=0)
     ui.add_ui_from_string(ui_string)
+    
+    # Add the old CTRL+R shortcut for legacy users
+    refresh = ui.get_widget('/menubar_main/menu_gwibber/refresh')
+    key, mod = gtk.accelerator_parse("<ctrl>R")
+    refresh.add_accelerator("activate", ui.get_accel_group(), key, mod, gtk.ACCEL_VISIBLE)
+
     return ui
 
   def update_menu_availability(self):
@@ -242,8 +321,18 @@ class GwibberClient(gtk.Window):
   def update_view(self):
     self.stream_view.update()
 
+  def private(self, message):
+    features = self.model.services[message["service"]]["features"]
+
+    if "private" in features:
+      self.message_target.set_target(message, "private")
+      self.input.textview.grab_focus()
+      buf = self.input.textview.get_buffer()
+      buf.place_cursor(buf.get_end_iter())
+
   def reply(self, message):
-    features = self.model.services[message["protocol"]]["features"]
+    features = self.model.services[message["service"]]["features"]
+
     if "reply" in features:
       if message["sender"].get("nick", 0) and not "thread" in features:
         s = "@%s: " if self.model.settings["reply_append_colon"] else "@%s "
@@ -258,55 +347,59 @@ class GwibberClient(gtk.Window):
     self.input.clear()
 
   def get_message(self, id):
-    return dict(self.model.messages.get_record(id).items())
+    with sqlite3.connect(SQLITE_DB_FILENAME) as db:
+      data = db.execute("SELECT data FROM messages WHERE id=?", (id,)).fetchone()
+    if data: return json.loads(data[0])
 
   def on_refresh(self, *args):
     self.service.Refresh()
 
-  def add_stream(self, data, kind=COUCH_TYPE_STREAM):
-    id = self.model.streams.put_record(CouchRecord(data, kind))
-    self.model.refresh()
-    self.service.PerformOp('{"id": "%s"}' % id)
-
+  def add_stream(self, data, kind=None):
     if "operation" in data:
       stream = str(self.model.features[data["operation"]]["stream"])
-    else: stream = "search"
+      obj = self.model.streams
+    else:
+      stream = "search"
+      obj = self.model.searches
 
+    id = obj.Create(json.dumps(data))
+    self.model.refresh()
     self.stream_view.new_stream({
       "stream": stream,
       "account": data.get("account", None),
       "transient": id,
     })
+
+    self.service.PerformOp('{"id": "%s"}' % id)
     self.update_menu_availability()
 
   def save_window_settings(self):
-    if str(LOCAL_SETTINGS["window_size"]) != str(self.get_size()):
-      self.gc.set_list(GCONF_CLIENT_DIR + "window_size", gconf.VALUE_INT, list(self.get_size()))
-    if str(LOCAL_SETTINGS["window_position"]) != str(self.get_position()):
-      self.gc.set_list(GCONF_CLIENT_DIR + "window_position", gconf.VALUE_INT, list(self.get_position()))
-    if str(LOCAL_SETTINGS["window_splitter"]) != str(self.input_splitter.get_position()):
-      self.gc.set_int(GCONF_CLIENT_DIR + "window_splitter",  int(self.input_splitter.get_position()))
-
+    self.model.settings["streams"] = json.dumps(self.stream_view.get_state())
+    self.model.settings["window_size"] = self.get_size()
+    self.model.settings["window_position"] = self.get_position()
+    self.model.settings["window_splitter"] = self.input_splitter.get_position()
+    
     if hasattr(self.stream_view, "splitter"):
-      if str(LOCAL_SETTINGS["sidebar_splitter"]) != str(self.stream_view.splitter.get_position()):
-        self.gc.set_int(GCONF_CLIENT_DIR + "sidebar_splitter",  int(self.stream_view.splitter.get_position()))
-
-    self.model.settings["streams"] =  self.stream_view.get_state()
-    self.model.settings.save()
+      self.model.settings["sidebar_splitter"] = self.stream_view.splitter.get_position()
 
   def on_pane_closed(self, widget, count):
     if count < 2 and isinstance(self.stream_view, gwui.MultiStreamUi):
       self.set_view("SingleStreamUi")
 
   def on_window_close(self, *args):
-    self.save_window_settings()
-    log.logger.info("Gwibber Client closed")
-    gtk.main_quit()
+    if self.model.settings["minimize_to_tray"]:
+      self.on_toggle_window_visibility(self)
+      return True
+    else:
+      self.save_window_settings()
+      log.logger.info("Gwibber Client closed")
+      gtk.main_quit()
 
   def on_quit(self, *args):
     self.service.Quit()
+    self.save_window_settings()
     log.logger.info("Gwibber Client quit")
-    self.on_window_close()
+    gtk.main_quit()
 
   def on_search(self, *args):
     self.stream_view.set_state([{
@@ -318,10 +411,7 @@ class GwibberClient(gtk.Window):
     self.stream_view.search_box.focus()
 
   def on_perform_search(self, widget, query):
-    self.add_stream({
-      "name": query,
-      "query": query,
-    }, COUCH_TYPE_SEARCH)
+    self.add_stream({"name": query, "query": query})
 
   def on_accounts(self, *args):
     if os.path.exists(os.path.join("bin", "gwibber-accounts")):
@@ -341,21 +431,24 @@ class GwibberClient(gtk.Window):
     self.ui.add_from_file (resources.get_ui_asset("gwibber-about-dialog.ui"))
     about_dialog = self.ui.get_object("about_dialog")
     about_dialog.set_version(str(VERSION_NUMBER))
+    about_dialog.set_transient_for(self)
     about_dialog.connect("response", lambda *a: about_dialog.hide())
     about_dialog.show_all()
 
   def on_close_stream(self, *args):
     state = self.stream_view.get_state()
     if state and state[0].get("transient", 0):
-      id = state[0]["transient"]
-      if self.model.streams.record_exists(id):
-        self.model.streams.delete_record(id)
-        self.stream_view.set_state([{"stream": "messages", "account": None}])
+      state = state[0]
+      obj = "searches" if state.get("stream", 0) == "search" else "streams"
+      getattr(self.model, obj).Delete(state["transient"])
+      self.stream_view.set_state([{"stream": "messages", "account": None}])
 
-  def on_message_action_menu(self, msg):
+  def on_message_action_menu(self, msg, view):
     theme = gtk.icon_theme_get_default()
     menu = gtk.Menu()
 
+    def perform_action(mi, act, widget, msg): act(widget, self, msg)
+
     for a in actions.MENU_ITEMS:
       if a.include(self, msg):
         image = gtk.image_new_from_icon_name(a.icon, gtk.ICON_SIZE_MENU)
@@ -363,21 +456,21 @@ class GwibberClient(gtk.Window):
         mi.set_label(a.label)
         mi.set_image(image)
         mi.set_property("use_underline", True)
-        mi.connect("activate", a.action, self, msg)
+        mi.connect("activate", perform_action, a.action, view, msg)
         menu.append(mi)
 
     menu.show_all()
     menu.popup(None, None, None, 3, 0)
 
-  def on_action(self, widget, uri, cmd, query):
+  def on_action(self, widget, uri, cmd, query, view):
     if hasattr(actions, cmd):
       if "msg" in query:
         query["msg"] = self.get_message(query["msg"])
-      getattr(actions, cmd).action(None, self, **query)
+      getattr(actions, cmd).action(view, self, **query)
 
   def on_stream_closed(self, widget, id, kind):
-    if self.model.streams.record_exists(id):
-      self.model.streams.delete_record(id)
+    self.model.streams.Delete(id)
+    self.model.searches.Delete(id)
 
   def on_stream_changed(self, widget, stream):
     self.update_menu_availability()
@@ -404,6 +497,9 @@ class GwibberClient(gtk.Window):
       elif action == "repost":
         data = {"message": text, "accounts": [target["account"]]}
         self.service.Send(json.dumps(data))
+      elif action == "private":
+        data = {"message": text, "private": target}
+        self.service.Send(json.dumps(data))
       self.message_target.end()
     else: self.service.SendMessage(text)
 
@@ -440,11 +536,7 @@ class Client(dbus.service.Object):
     dbus.service.Object.__init__(self, self.bus_name, self.__dbus_object_path__)
 
     # Methods the client exposes via dbus, return from the list method
-    self.exposed_methods = [
-                       'focus_client',
-                       'show_replies',
-                      ]
-
+    self.exposed_methods = ["focus_client", "show_replies"]
     self.w = GwibberClient()
 
   @dbus.service.method("com.GwibberClient", in_signature="", out_signature="")
@@ -467,9 +559,8 @@ class Client(dbus.service.Object):
     """
     self.w.present_with_time(int(time.mktime(datetime.datetime.now().timetuple())))
     self.w.move(*self.w.model.settings["window_position"])
-    # FIXME: we need to be able to select the stream to view
-    #self.w.account_tree.get_selection().unselect_all()
-    #self.w.account_tree.get_selection().select_path((2,))
+    stream = {'account': None, 'stream': 'replies', 'transient': False}
+    self.w.stream_view.set_state([stream])
 
   @dbus.service.method("com.GwibberClient")
   def list(self):
