@@ -1,7 +1,8 @@
 
-import os, json, urlparse, resources, util
+import os, json, urlparse, resources, util, sqlite3
 import gtk, gobject, pango, webkit, time, gconf
-import gwibber.microblog.util
+import microblog.util
+from microblog import config
 
 import gettext
 from gettext import lgettext as _
@@ -13,10 +14,12 @@ from mako.template import Template
 from mako.lookup import TemplateLookup
 
 from microblog.util.const import *
-from microblog.util.couch import Monitor as CouchMonitor
-
-from desktopcouch.records.server import CouchDatabase
-from desktopcouch.records.record import Record as CouchRecord
+# Try to import * from custom, install custom.py to include packaging 
+# customizations like distro API keys, etc
+try:
+  from microblog.util.custom import *
+except:
+  pass
 
 gtk.gdk.threads_init()
 
@@ -30,22 +33,18 @@ class Model(gobject.GObject):
   def __init__(self):
     gobject.GObject.__init__(self)
 
-    self.daemon = gwibber.microblog.util.getbus("Service")
+    self.settings = config.Preferences()
+    self.daemon = microblog.util.getbus("Service")
+    self.accounts = microblog.util.getbus("Accounts")
+    self.streams = microblog.util.getbus("Streams")
+    self.searches = microblog.util.getbus("Searches")
 
-    self.account_monitor = gwibber.microblog.util.getbus("Accounts")
-    self.account_monitor.connect_to_signal("AccountChanged", self.on_stream_changed)
-    self.account_monitor.connect_to_signal("AccountDeleted", self.on_stream_changed)
-
-    self.streams_monitor = gwibber.microblog.util.getbus("Streams")
-    self.streams_monitor.connect_to_signal("StreamChanged", self.on_stream_changed)
-    self.streams_monitor.connect_to_signal("StreamClosed", self.on_stream_changed)
-
-    self.settings = gwibber.microblog.util.SettingsMonitor()
     self.services = json.loads(self.daemon.GetServices())
     self.features = json.loads(self.daemon.GetFeatures())
-    self.messages = CouchDatabase(COUCH_DB_MESSAGES, create=True)
-    self.accounts = CouchDatabase(COUCH_DB_ACCOUNTS, create=True)
-    self.streams = CouchDatabase(COUCH_DB_SETTINGS, create=True)
+
+    for b in [self.accounts, self.searches, self.streams]:
+      for i in ["Created", "Updated", "Deleted"]:
+        b.connect_to_signal(i, self.on_stream_changed)
 
     self.model = None
     self.model_valid = False
@@ -85,143 +84,119 @@ class Model(gobject.GObject):
 
   def generate_streams(self):
     items = []
-    transients = self.streams.get_records(COUCH_TYPE_STREAM, True)
+    transients = json.loads(self.streams.List())
 
     items.append({
       "name": _("Home"),
       "stream": "home",
-      "view": self.messages.execute_view("home", "messages")[[{}]:[0]],
+      "sql": "select data from messages where transient = 0",
       "account": None,
       "transient": False,
       "color": None,
     })
 
-    items.append({
-      "name": _("Messages"),
-      "stream": "messages",
-      "view": self.messages.execute_view("stream_time", "messages")[["messages", {}]:["messages", 0]],
-      "account": None,
-      "transient": False,
-      "color": None,
-    })
-
-    items.append({
-      "name": _("Replies"),
-      "stream": "replies",
-      "view": self.messages.execute_view("stream_time", "messages")[["replies", {}]:["replies", 0]],
-      "account": None,
-      "transient": False,
-      "color": None,
-    })
-
-    items.append({
-      "name": _("Images"),
-      "stream": "images",
-      "view": self.messages.execute_view("stream_time", "messages")[["images", {}]:["images", 0]],
-      "account": None,
-      "transient": False,
-      "color": None,
-    })
-
-    items.append({
-      "name": _("Private"),
-      "stream": "private",
-      "view": self.messages.execute_view("stream_time", "messages")[["private", {}]:["private", 0]],
-      "account": None,
-      "transient": False,
-      "color": None,
-    })
+    # This list is defined just to get the strings in the template for translation
+    stream_titles = [_("Messages"), _("Replies"), _("Images"), _("Private")]
 
+    for stream in ["messages", "replies", "images", "private"]:
+      items.append({
+        "name": _(stream.capitalize()),
+        "stream": stream,
+        "sql": "SELECT data FROM messages WHERE stream = '%s' AND transient = 0" % stream,
+        "account": None,
+        "transient": False,
+        "color": None,
+      })
 
     items.append({
       "name": _("Sent"),
       "stream": "sent",
-      "view": self.messages.execute_view("mine", "messages")[[{}]:[0]],
+      "sql": "SELECT data FROM messages WHERE from_me = 1 AND transient = 0",
       "account": None,
       "transient": False,
       "color": None,
     })
 
-    for account in self.accounts.get_records(COUCH_TYPE_ACCOUNT, True):
-      account = account.value
-      aId = account["_id"]
+    for account in json.loads(self.accounts.List()):
+      aId = account["id"]
 
-      item = {
-          "name": account.get("username", "None"),
-          "account": aId,
-          "stream": None,
-          "view": self.messages.execute_view("account_time", "messages")[[aId, {}]:[aId, 0]],
-          "transient": False,
-          "color": util.Color(account["color"]),
-          "protocol": account["protocol"],
-          "items": [],
-      }
+      if self.services.has_key(account["service"]) and "receive" in self.services[account["service"]]["features"]:
 
-      default_streams = self.services[account["protocol"]]["default_streams"]
-
-      if len(default_streams) > 1:
-        for feature in default_streams:
-          aname = self.features[feature]["stream"]
-          item["items"].append({
-            "name": _(aname.capitalize()),
+        item = {
+            "name": account.get("username", "None"),
             "account": aId,
-            "stream": aname,
-            "view": self.messages.execute_view("account_stream_time", "messages")[[aId, aname, {}]:[aId, aname, 0]],
+            "stream": None,
+            "sql": "SELECT data FROM messages WHERE account = '%s' AND transient = 0" % aId,
             "transient": False,
             "color": util.Color(account["color"]),
-            "protocol": account["protocol"],
-          })
+            "service": account["service"],
+            "items": [],
+        }
 
-        item["items"].append({
-          "name": "Sent",
-          "account": aId,
-          "stream": "sent",
-          "view": self.messages.execute_view("mine_account", "messages")[[aId, {}]:[aId, 0]],
-          "transient": False,
-          "color": util.Color(account["color"]),
-          "protocol": account["protocol"],
-        })
+        default_streams = self.services[account["service"]]["default_streams"]
 
-      for transient in transients:
-        transient = transient.value
-        tId = transient["_id"]
-
-        if transient["account"] == aId:
-          if transient["operation"] == "user_messages" and account["protocol"] in ["twitter", "identica"]:
-            uId = transient["parameters"]["id"]
-            view = self.messages.execute_view("user_protocol_time", "messages")[[uId, account["protocol"], {}]:[uId, account["protocol"], 0]]
-          else:
-            view = self.messages.execute_view("transient_time", "messages")[[tId, {}]:[tId, 0]]
+        if len(default_streams) > 1:
+          for feature in default_streams:
+            aname = self.features[feature]["stream"]
+            item["items"].append({
+              "name": _(aname.capitalize()),
+              "account": aId,
+              "stream": aname,
+              "sql": "SELECT data FROM messages WHERE account = '%s' AND stream = '%s' AND transient = 0" % (aId, aname),
+              "transient": False,
+              "color": util.Color(account["color"]),
+              "service": account["service"],
+            })
 
           item["items"].append({
-            "name": transient["name"],
+            "name": _("Sent"),
             "account": aId,
-            "stream": self.features[transient["operation"]]["stream"],
-            "view": view,
-            "transient": tId,
+            "stream": "sent",
+            "sql": "SELECT data FROM messages WHERE account = '%s' AND from_me = 1 AND transient = 0" % aId,
+            "transient": False,
             "color": util.Color(account["color"]),
-            "protocol": account["protocol"],
+            "service": account["service"],
           })
 
-      items.append(item)
+        for transient in transients:
+          tId = transient["id"]
+
+          if transient["account"] == aId:
+            query = "SELECT data FROM messages WHERE transient = '%s'" % tId
+
+            if transient["operation"] == "user_messages" and account["service"] in ["twitter", "identica"]:
+              query += (" OR (account = '%s' AND recipient = '%s')" % (aId, transient["parameters"]["id"]))
+
+            if transient["name"] == "Lists": transient["name"] = _("Lists")
+
+            item["items"].append({
+              "name": transient["name"],
+              "account": aId,
+              "stream": self.features[transient["operation"]]["stream"],
+              "sql": query,
+              "transient": tId,
+              "color": util.Color(account["color"]),
+              "service": account["service"],
+            })
+
+        items.append(item)
 
     searches = {
-        "name": "Search",
+        "name": _("Search"),
         "account": None,
         "stream": "search",
-        "view": self.messages.execute_view("search_time", "messages")[{}:0],
+        "sql": "SELECT data FROM messages WHERE stream = 'search'",
         "transient": False,
         "color": None,
         "items": [],
     }
 
-    for search in self.streams.get_records(COUCH_TYPE_SEARCH, True):
-      search = search.value
-      sId = search["_id"]
+    for search in json.loads(self.searches.List()):
+      sId = search["id"]
       searches["items"].append({
         "name": search["name"],
         "account": None,
-        "view": self.messages.execute_view("transient_time", "messages")[[sId, {}]:[sId, 0]],
+        "sql": "SELECT data FROM messages WHERE transient = '%s'" % sId,
         "stream": "search",
         "transient": sId,
         "color": None,
@@ -239,6 +214,7 @@ class WebUi(webkit.WebView):
     webkit.WebView.__init__(self)
     self.web_settings = webkit.WebSettings()
     self.set_settings(self.web_settings)
+    self.web_settings.set_property("enable-plugins", False)
     self.gc = gconf.client_get_default()
 
     self.connect("navigation-requested", self.on_click_link)
@@ -262,10 +238,11 @@ class WebUi(webkit.WebView):
     return True
 
   def render(self, theme, template, **kwargs):
-    default_font = self.gc.get_string("/desktop/gnome/interface/font_name")
-    font_name, font_size = default_font.rsplit(None, 1)
-    self.web_settings.set_property("sans-serif-font-family", font_name)
-    self.web_settings.set_property("default-font-size", float(font_size))
+    default_font = self.gc.get_string("/desktop/gnome/interface/document_font_name")
+    if default_font:
+      font_name, font_size = default_font.rsplit(None, 1)
+      self.web_settings.set_property("sans-serif-font-family", font_name)
+      self.web_settings.set_property("default-font-size", float(font_size))
 
     if not resources.theme_exists(theme):
       theme = "default"
@@ -323,7 +300,7 @@ class Navigation(WebUi):
 
 class SingleStreamUi(gtk.VBox):
   __gsignals__ = {
-    "action": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, (str, str, object)),
+    "action": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, (str, str, object, object)),
     "stream-closed": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, (str, str)),
     "stream-changed": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, (object,)),
     "search": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, (str,)),
@@ -344,32 +321,22 @@ class SingleStreamUi(gtk.VBox):
     self.navigation_scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_NEVER)
     self.navigation_scroll.add(self.navigation)
 
-    # Build the main message view
-    self.message_view = MessageStreamView(self.model)
-    self.message_view.connect("action", self.on_action)
-
+    self.messages = MessageStream(self.model)
+    self.messages.message_view.connect("action", self.on_action)
+    
     self.search_box = GwibberSearch()
     self.search_box.connect("search", self.on_search)
-
-    self.message_scroll = gtk.ScrolledWindow()
-    self.message_scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-    self.message_scroll.set_shadow_type(gtk.SHADOW_IN)
-    self.message_scroll.add(self.message_view)
-
+    
     layout = gtk.VBox(spacing=5)
     layout.pack_start(self.search_box, False)
-    layout.pack_start(self.message_scroll, True)
+    layout.pack_start(self.messages, True)
 
     # Build the pane layout
     self.splitter = gtk.HPaned()
     self.splitter.add1(self.navigation_scroll)
     self.splitter.add2(layout)
 
-    self.sidebar_splitter = self.gc.get_int(GCONF_CLIENT_DIR + "sidebar_splitter") or LOCAL_SETTINGS["sidebar_splitter"]
-    self.splitter.set_position(self.sidebar_splitter)
-    self.handle_splitter_position_change(self.sidebar_splitter)
     self.splitter.connect("notify", self.on_splitter_drag)
-
     self.pack_start(self.splitter, True)
 
   def handle_splitter_position_change(self, pos):
@@ -413,19 +380,18 @@ class SingleStreamUi(gtk.VBox):
   def update_search_visibility(self):
     stream = self.navigation.selected_stream
     if stream is not None:
-      is_search = stream["stream"] == "search" and stream["name"] == "Search"
+      is_search = stream["stream"] == "search" and not stream["transient"] 
       self.search_box.set_visible(not is_search)
 
   def on_action(self, widget, uri, cmd, query):
-    self.emit("action", uri, cmd, query)
+    self.emit("action", uri, cmd, query, self)
 
   def on_search(self, widget, query):
     self.emit("search", query)
 
   def update(self, *args):
-    if self.navigation.selected_stream:
-      self.message_view.render([self.navigation.selected_stream["view"]])
-
+    self.messages.update(self.navigation.selected_stream)
+    
   def get_state(self):
     return [self.model.to_state(self.navigation.selected_stream)]
 
@@ -433,15 +399,16 @@ class SingleStreamUi(gtk.VBox):
     if state: self.set_state([state])
 
   def set_state(self, streams):
-    self.navigation.selected_stream = self.model.find(**streams[0]) or self.model.find(stream="home", account=None)
-    self.navigation.render()
-    self.update_search_visibility()
-    self.update()
+    if streams:
+      self.navigation.selected_stream = self.model.find(**streams[0]) or self.model.find(stream="home", account=None)
+      self.navigation.render()
+      self.update_search_visibility()
+      self.update()
 
 
 class MultiStreamUi(gtk.HBox):
   __gsignals__ = {
-    "action": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, (str, str, object)),
+    "action": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, (str, str, object, object)),
     "stream-closed": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, (str, str)),
     "stream-changed": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, (object,)),
     "pane-closed": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, (int,)),
@@ -471,7 +438,7 @@ class MultiStreamUi(gtk.HBox):
     self.emit("search", query)
 
   def on_action(self, widget, uri, cmd, query):
-    self.emit("action", uri, cmd, query)
+    self.emit("action", uri, cmd, query, widget)
 
   def new_stream(self, state={"stream": "messages", "account": None}):
     item = MultiStreamPane(self.model)
@@ -543,17 +510,13 @@ class MultiStreamPane(gtk.VBox):
     self.navigation_bar.pack_end(btn_close, False)
 
     # Build the main message view
-    self.message_view = MessageStreamView(self.model)
-    self.message_view.connect("action", self.on_action)
 
-    self.message_scroll = gtk.ScrolledWindow()
-    self.message_scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-    self.message_scroll.set_shadow_type(gtk.SHADOW_IN)
-    self.message_scroll.add(self.message_view)
+    self.messages = MessageStream(self.model)
+    self.messages.message_view.connect("action", self.on_action)
 
     self.pack_start(self.navigation_bar, False)
     self.pack_start(self.search_box, False)
-    self.pack_start(self.message_scroll, True)
+    self.pack_start(self.messages, True)
 
   def on_close(self, *args):
     self.emit("pane-closed")
@@ -595,11 +558,11 @@ class MultiStreamPane(gtk.VBox):
     self.selected_stream = stream
     self.nav_label.set_text(stream["name"])
 
-    is_search = stream["stream"] == "search" and stream["name"] == "Search"
+    is_search = stream["stream"] == "search" and not stream["transient"]
     self.search_box.set_visible(not is_search)
 
     if stream["account"]:
-      fname = resources.get_ui_asset("icons/breakdance/16x16/%s.png" % stream["protocol"])
+      fname = resources.get_ui_asset("icons/breakdance/16x16/%s.png" % stream["service"])
       self.icon_protocol.set_from_file(fname)
     else: self.icon_protocol.clear()
 
@@ -616,7 +579,7 @@ class MultiStreamPane(gtk.VBox):
 
   def update(self, *args):
     if self.selected_stream:
-      self.message_view.render([self.selected_stream["view"]])
+      self.messages.update(self.selected_stream)
 
   def get_state(self):
     return self.model.to_state(self.selected_stream)
@@ -649,7 +612,7 @@ class AccountTargetBar(gtk.HBox):
 
   def set_target(self, message, action="reply"):
     self.target = message
-    self.target["account_data"] = dict(self.model.accounts.get_record(message["account"]).items())
+    self.target["account_data"] = json.loads(self.model.accounts.Get(message["account"]))
     self.action = action
     self.render()
 
@@ -661,12 +624,12 @@ class AccountTargetBar(gtk.HBox):
 
   def on_action(self, w, uri, cmd, query):
     if cmd == "cancel": return self.end()
-    if cmd == "account" and "id" in query and \
-        self.model.accounts.record_exists(query["id"]):
-
-      if "send_enabled" in query:
-        self.model.accounts.update_fields(query["id"],
-            {"send_enabled": bool(query["send_enabled"] == "true")})
+    if cmd == "account" and "id" in query:
+      acct = self.model.accounts.Get(query["id"])
+      if acct and "send_enabled" in query:
+        acct = json.loads(acct)
+        acct["send_enabled"] = bool(query["send_enabled"] == "true")
+        self.model.accounts.Update(json.dumps(acct))
 
   def on_account_changed(self, id):
     self.populate()
@@ -674,9 +637,9 @@ class AccountTargetBar(gtk.HBox):
 
   def populate(self):
     self.accounts = []
-    for account in self.model.accounts.get_records(COUCH_TYPE_ACCOUNT, True):
-      if "send" in self.model.services[account.value["protocol"]]["features"]:
-        self.accounts.append(account.value)
+    for account in json.loads(self.model.accounts.List()):
+      if self.model.services.has_key(account["service"]) and "send" in self.model.services[account["service"]]["features"]:
+        self.accounts.append(account)
 
   def render(self):
     return self.targetbar.render(self.model.settings["theme"], "targetbar.mako",
@@ -685,38 +648,118 @@ class AccountTargetBar(gtk.HBox):
         action=self.action,
         accounts=self.accounts)
 
+class MessageStream(gtk.HBox):
+  def __init__(self, model):
+    gtk.HBox.__init__(self, spacing=5)
+
+    self.previous_threshold = 100
+    self.previous_position = 0
+    self.previous_stream = None
+    self.messages = None
+
+    self.add_events(gtk.gdk.SCROLL_MASK)
+    self.connect("scroll-event", self.on_viewport_scroll)
+
+    self.message_view = MessageStreamView(model)
+    self.message_view.connect("load-finished", self.on_scroll)
+
+    self.message_scroll = gtk.ScrolledWindow()
+    self.message_scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_NEVER)
+    self.message_scroll.set_shadow_type(gtk.SHADOW_IN)
+    self.message_scroll.add(self.message_view)
+
+    self.scrollbar = gtk.VScrollbar()
+    self.scrollbar.set_range(0, 100)
+    self.scrollbar.set_increments(1, 10)
+    self.scrollbar.connect("value-changed", self.on_scroll)
+
+    self.pack_start(self.message_scroll, True, True)
+    self.pack_start(self.scrollbar, False)
+
+  def on_viewport_scroll(self, widget, event):
+    value = int(self.scrollbar.get_value())
+
+    if event.direction == gtk.gdk.SCROLL_DOWN: value += 1
+    elif event.direction == gtk.gdk.SCROLL_UP: value -= 1
+
+    self.scrollbar.set_value(value)
+    self.handle_scroll(value)
+
+  def on_scroll(self, *args):
+    self.handle_scroll(int(self.scrollbar.get_value()))
+
+  def handle_scroll(self, value):
+    self.message_view.execute_script("""
+    document.location.hash = "";
+    document.location.hash = "msg-%s"
+    """ % value)
+
+    self.previous_position = value
+
+    return
+
+    ### Experimenting with infinite scrolling
+
+    threshold = len(self.messages) - 6
+
+    if value > threshold and len(self.messages) < self.previous_threshold:
+      print "Loading more messages"
+      self.previous_threshold  += 100
+      self.update(self.previous_stream, self.previous_threshold)
+
+    self.previous_position = value
+
+  def update(self, selected_stream, count=100):
+    id = None
+    
+    if selected_stream:
+      same_stream = selected_stream == self.previous_stream
+      self.previous_stream = selected_stream 
+      
+      if self.messages and same_stream:
+        index = int(self.scrollbar.get_value())
+        if index < len(self.messages) and index > 0:
+          id = self.messages[index]
+
+      self.messages = self.message_view.render([selected_stream], count)
+      pos = self.messages.index(id) if id and id in self.messages else 0
+      self.scrollbar.set_range(0, len(self.messages) - 1)
+      self.scrollbar.set_value(pos)
+
 class MessageStreamView(WebUi):
   def __init__(self, model):
     WebUi.__init__(self)
     self.model = model
 
-  def render(self, views):
-    accounts = CouchDatabase(COUCH_DB_ACCOUNTS).get_records(COUCH_TYPE_ACCOUNT, True)
-
-    accounts = dict((a.id, a.value) for a in accounts)
+  def render(self, streams, count = 100):
+    accounts = json.loads(self.model.accounts.List())
+    accounts = dict((a["id"], a) for a in accounts)
     messages = []
     seen = {}
 
-    for view in views:
-      view.options["descending"] = True
-      view.options["limit"] = 100
-      view.options["include_docs"] = True
-      view._fetch()
-
-      for item in view:
-        message = item['doc']
+    if len(streams) == 1 and "sql" in streams[0]:
+      with sqlite3.connect(SQLITE_DB_FILENAME) as db:
+        data = db.execute("%s order by time desc limit %s" % (streams[0]["sql"], count)).fetchall()
+        msgs = json.loads("[%s]" % ",".join(i[0] for i in data))
+      
+      for item in msgs:
+        message = item
         message["dupes"] = []
         message["txtid"] = util.remove_urls(message["text"]).strip()[:MAX_MESSAGE_LENGTH] or None
         message["color"] = util.Color(accounts.get(message["account"], {"color": "#5A5A5A"})["color"])
         message["time_string"] = util.generate_time_string(message["time"])
+        try:
+          message["sender"]["image"] = resources.get_avatar_path(message["sender"]["image"])
+        except:
+          pass
         messages.append(message)
 
     def dupematch(item, message):
-      if item["protocol"] == message["protocol"] and item["id"] == message["id"]:
+      if item["service"] == message["service"] and item["id"] == message["id"]:
         return True
 
       for item in item["dupes"]:
-        if item["protocol"] == message["protocol"] and item["id"] == message["id"]:
+        if item["service"] == message["service"] and item["id"] == message["id"]:
           return True
 
     # Detect duplicates
@@ -730,12 +773,16 @@ class MessageStreamView(WebUi):
         if message["txtid"]:
           seen[message["txtid"]] = n
 
+    messages = [m for m in messages if not m["is_dupe"]]
+
     WebUi.render(self, self.model.settings["theme"], "template.mako",
         message_store=messages,
         preferences=self.model.settings,
         services=self.model.services,
         accounts=accounts)
 
+    return [m["txtid"] for m in messages]
+
 class GwibberSearch(gtk.HBox):
   __gsignals__ = {
     "search": (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION, None, (str,)),
@@ -833,9 +880,9 @@ class InputTextView(gtk.TextView):
     self.overlay_color = util.get_theme_colors()["text"].darker(3).hex
     self.overlay_text = '<span weight="bold" size="xx-large" foreground="%s">%s</span>'
 
-    self.shortener = gwibber.microblog.util.getbus("URLShorten")
+    self.shortener = microblog.util.getbus("URLShorten")
 
-    self.connection = gwibber.microblog.util.getbus("Connection")
+    self.connection = microblog.util.getbus("Connection")
     self.connection.connect_to_signal("ConnectionOnline", self.on_connection_online)
     self.connection.connect_to_signal("ConnectionOffline", self.on_connection_offline)
 
@@ -907,6 +954,8 @@ class InputTextView(gtk.TextView):
             error_handler=error_shortened)
 
   def set_overlay_text(self, text):
+    if not self.pango_overlay:
+      self.pango_overlay = self.create_pango_layout("")
     self.pango_overlay.set_markup(self.overlay_text % (self.overlay_color, text))
 
   def on_size_allocate(self, *args):
@@ -928,10 +977,10 @@ class InputTextView(gtk.TextView):
     color = self.error_color if chars > MAX_MESSAGE_LENGTH else self.base_color
     self.modify_base(gtk.STATE_NORMAL, color)
 
-  def on_connection_online(self, w):
+  def on_connection_online(self, *args):
     self.set_sensitive(True)
 
-  def on_connection_offline(self, w):
+  def on_connection_offline(self, *args):
     self.set_sensitive(False)
 
 gtk.binding_entry_add_signal(InputTextView, gtk.keysyms.Return, 0, "submit")
