@@ -22,14 +22,31 @@ import gtk
 import urllib
 import webkit
 import string
+
 from gtk import Builder
 import gwibber.microblog
 from gwibber.microblog import facebook
 from gwibber.microblog.util import facelib
 from gwibber.microblog.util.const import *
-import json, urlparse, gnomekeyring, uuid
-from gettext import gettext as _
+# Try to import * from custom, install custom.py to include packaging 
+# customizations like distro API keys, etc
+try:
+  from gwibber.microblog.util.custom import *
+except:
+  pass
 
+import json, urlparse, gnomekeyring, uuid
+import gettext
+from gettext import gettext as _
+if hasattr(gettext, 'bind_textdomain_codeset'):
+    gettext.bind_textdomain_codeset('gwibber','UTF-8')
+gettext.textdomain('gwibber')
+
+"""
+gtk.gdk.threads_init()
+
+APP_KEY = "71b85c6d8cb5bbb9f1a3f8bbdcdd4b05"
+"""
 
 class AccountWidget(gtk.VBox):
   """AccountWidget: A widget that provides a user interface for configuring facebook accounts in Gwibber
@@ -44,16 +61,16 @@ class AccountWidget(gtk.VBox):
     self.ui.connect_signals(self)
     self.vbox_settings = self.ui.get_object("vbox_settings")
     self.pack_start(self.vbox_settings, False, False)
-    self.vbox_settings.show_all()
+    self.show_all()
     if account:
       self.account = account
     else:
       self.account = {}
     self.dialog = dialog
     has_secret_key = True
-    if self.account.has_key("_id"):
+    if self.account.has_key("id"):
       try:
-        value = gnomekeyring.find_items_sync(gnomekeyring.ITEM_GENERIC_SECRET, {"id": str("%s/%s" % (self.account["_id"], "secret_key"))})[0].secret
+        value = gnomekeyring.find_items_sync(gnomekeyring.ITEM_GENERIC_SECRET, {"id": str("%s/%s" % (self.account["id"], "secret_key"))})[0].secret
       except gnomekeyring.NoMatchError:
         has_secret_key = False
     try:
@@ -65,7 +82,7 @@ class AccountWidget(gtk.VBox):
         self.ui.get_object("hbox_facebook_auth_done").hide()
         self.ui.get_object("facebook_auth_button").modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("red"))
         if self.dialog:
-            self.dialog.get_object('vbox_create').hide()
+          self.dialog.get_object('vbox_create').hide()
     except:
       self.ui.get_object("hbox_facebook_auth_done").hide()
       if self.dialog:
@@ -75,6 +92,7 @@ class AccountWidget(gtk.VBox):
     (self.win_w, self.win_h) = self.window.get_size()
 
     web = webkit.WebView()
+    web.get_settings().set_property("enable-plugins", False)
     web.load_html_string(_("<p>Please wait...</p>"), "file:///")
 
     url = urllib.urlencode({
@@ -111,9 +129,12 @@ class AccountWidget(gtk.VBox):
         fbc.session_key = self.account["session_key"]
         fbc.secret_key = str(data["secret"])
         self.account["username"] = str(fbc.users.getInfo(fbuid)[0]["name"])
+        self.account["secret_key"] = str(data["secret"])
+        
         if not self.account.has_key("username"):
             self.account["username"] = str(fbc.users.getInfo(fbuid)[0]["uid"])
 
+        """
         if "_id" not in self.account:
           valid = string.ascii_letters + string.digits + "-"
           aId = "facebook-%s" % self.account["username"]
@@ -126,11 +147,14 @@ class AccountWidget(gtk.VBox):
                 "Gwibber pref: %s/%s" % (self.account["_id"], "secret_key"),
                 {"id": str("%s/%s" % (self.account["_id"], "secret_key"))},
                 str(data["secret"]), True)
-        
+        """
+
         self.ui.get_object("hbox_facebook_auth").hide()
         self.ui.get_object("fb_auth_done_label").set_label(_("%s has been authorized by Facebook") % str(self.account["username"]))
         self.ui.get_object("hbox_facebook_auth_done").show()
-        if self.dialog:
+        if self.dialog and self.account.has_key("id"):
+          self.dialog.get_object("vbox_save").show()
+        elif self.dialog:
           self.dialog.get_object("vbox_create").show()
       except:
         #FIXME: We should do this in the same window
