@@ -1,13 +1,16 @@
 import gtk, webkit, gnomekeyring
 import urllib, urllib2, json, urlparse, uuid
 from oauth import oauth
-import string
 
 from gtk import Builder
 import gwibber.microblog
 from gwibber.microblog import twitter
 from gwibber.microblog.util import resources
+import gettext
 from gettext import gettext as _
+if hasattr(gettext, 'bind_textdomain_codeset'):
+    gettext.bind_textdomain_codeset('gwibber','UTF-8')
+gettext.textdomain('gwibber')
 
 gtk.gdk.threads_init()
 
@@ -31,11 +34,9 @@ class AccountWidget(gtk.VBox):
     self.account = account or {}
     self.dialog = dialog
     has_secret_key = True
-    self.is_new = True
-    if self.account.has_key("_id"):
-      self.is_new = False
+    if self.account.has_key("id"):
       try:
-        value = gnomekeyring.find_items_sync(gnomekeyring.ITEM_GENERIC_SECRET, {"id": str("%s/%s" % (self.account["_id"], "secret_token"))})[0].secret
+        value = gnomekeyring.find_items_sync(gnomekeyring.ITEM_GENERIC_SECRET, {"id": str("%s/%s" % (self.account["id"], "secret_token"))})[0].secret
       except gnomekeyring.NoMatchError:
         has_secret_key = False
 
@@ -59,6 +60,7 @@ class AccountWidget(gtk.VBox):
     self.winsize = self.window.get_size()
 
     web = webkit.WebView()
+    web.get_settings().set_property("enable-plugins", False)
     web.load_html_string(_("<p>Please wait...</p>"), "file:///")
 
     self.consumer = oauth.OAuthConsumer(*resources.get_twitter_keys())
@@ -92,7 +94,7 @@ class AccountWidget(gtk.VBox):
       try:
         url = web.get_main_frame().get_uri()
         data = urlparse.parse_qs(url.split("?", 1)[1])
-      
+        
         token = data["oauth_token"][0]
         verifier = data["oauth_verifier"][0]
 
@@ -106,31 +108,17 @@ class AccountWidget(gtk.VBox):
         data = urlparse.parse_qs(tokendata)
 
         self.account["access_token"] = data["oauth_token"][0]
+        self.account["secret_token"] = data["oauth_token_secret"][0]
         self.account["username"] = data["screen_name"][0]
         self.account["user_id"] = data["user_id"][0]
 
-        if "_id" not in self.account:
-          valid = string.ascii_letters + string.digits + "-"
-          aId = "twitter-%s" % self.account["username"]
-          self.account["_id"] = "".join((x for x in aId if x in valid)).lower()
-
-        self.account["secret_token"] = ":KEYRING:%s" % \
-          gnomekeyring.item_create_sync(
-            gnomekeyring.get_default_keyring_sync(),
-            gnomekeyring.ITEM_GENERIC_SECRET,
-            "Gwibber pref: %s/%s" % (self.account["_id"], "secret_token"),
-            {"id": str("%s/%s" % (self.account["_id"], "secret_token"))},
-           str(data["oauth_token_secret"][0]), True)
-
-        if self.account.has_key("password"): self.account["password"] = "DELETED"
-
         self.ui.get_object("hbox_twitter_auth").hide()
         self.ui.get_object("fb_auth_done_label").set_label(_("%s has been authorized by Twitter") % str(self.account["username"]))
         self.ui.get_object("hbox_twitter_auth_done").show()
-        if self.dialog and self.is_new:
-          self.dialog.get_object("vbox_create").show()
-        elif self.dialog:
+        if self.dialog and self.account.has_key("id"):
           self.dialog.get_object("vbox_save").show()
+        elif self.dialog:
+          self.dialog.get_object("vbox_create").show()
       except:
         pass
 
