@@ -31,9 +31,9 @@ class Client:
 
   def _story(self, data):
     m = {}; 
-    m["id"] = str(data["id"])
-    m["protocol"] = "digg"
-    m["account"] = self.account["_id"]
+    m["mid"] = str(data["id"])
+    m["service"] = "digg"
+    m["account"] = self.account["id"]
     m["time"] = data["submit_date"]
 
     m["text"] = data["title"] + "\n" + data["description"]
