@@ -37,9 +37,9 @@ class Client:
 
   def _message(self, data):
     m = {}
-    m["id"] = str(data["id"])
-    m["protocol"] = "flickr"
-    m["account"] = self.account["_id"]
+    m["mid"] = str(data["id"])
+    m["service"] = "flickr"
+    m["account"] = self.account["id"]
     m["time"] = parse_time(data["dateupload"])
     m["source"] = False
     m["text"] = data["title"]
