@@ -56,9 +56,9 @@ class Client:
 
   def _message(self, data):
     m = {
-      "id": data["id"],
-      "protocol": "friendfeed",
-      "account": self.account["_id"],
+      "mid": data["id"],
+      "service": "friendfeed",
+      "account": self.account["id"],
       "time": util.parsetime(data["published"]),
       "source": data.get("via", {}).get("name", None),
       "text": data["title"],
@@ -69,7 +69,7 @@ class Client:
     }
     
     if data.get("service", 0):
-      m["service"] = {
+      m["origin"] = {
         "id": data["service"]["id"],
         "name": data["service"]["name"],
         "icon": data["service"]["iconUrl"],
@@ -100,18 +100,16 @@ class Client:
     return m
 
   def _get(self, path, post=False, parse="message", kind="entries", single=False, **args):
-    passwd = self.account.get("secret_key", self.account["password"])
-    
     url = "/".join((URL_PREFIX, path))
     data = network.Download(url, util.compact(args), post,
-        self.account["username"], passwd).get_json()
+        self.account["username"], self.account["secret_key"]).get_json()
 
     if isinstance(data, dict) and data.get("errorCode", 0):
       if "unauthorized" in data["errorCode"]:
-        raise exceptions.GwibberProtocolError("auth", self.account["protocol"], self.account["username"], data["errorCode"])
+        raise exceptions.GwibberServiceError("auth", self.account["service"], self.account["username"], data["errorCode"])
    
     if single: return [getattr(self, "_%s" % parse)(data)]
-    if parse: return [getattr(self, "_%s" % parse)(m) for m in data]
+    if parse: return [getattr(self, "_%s" % parse)(m) for m in data["entries"]]
     else: return []
 
     if parse:
@@ -140,11 +138,11 @@ class Client:
     return self._get("feed/user/%s" % id, num=count, start=since)
 
   def delete(self, message):
-    self._get("entry/delete", True, None, entry=message["id"])
+    self._get("entry/delete", True, None, entry=message["mid"])
     return []
 
   def like(self, message):
-    self._get("entry/like", True, None, entry=message["id"])
+    self._get("entry/like", True, None, entry=message["mid"])
     return []
 
   def send(self, message):
@@ -152,6 +150,6 @@ class Client:
     return []
 
   def send_thread(self, message, target):
-    self._get("comment", True, None, body=message, entry=target["id"])
+    self._get("comment", True, None, body=message, entry=target["mid"])
     return []
 
