@@ -32,6 +32,7 @@ PROTOCOL_INFO = {
     "retweet",
     "like",
     "send_thread",
+    "send_private",
     "user_messages",
     "sinceid",
   ],
@@ -52,31 +53,36 @@ class Client:
   def _common(self, data):
     m = {}
     try:
-      m["id"] = str(data["id"])
-      m["protocol"] = "identica"
-      m["account"] = self.account["_id"]
+      m["mid"] = str(data["id"])
+      m["service"] = "identica"
+      m["account"] = self.account["id"]
       m["time"] = util.parsetime(data["created_at"])
       m["source"] = data.get("source", False)
       m["text"] = data["text"]
       m["to_me"] = ("@%s" % self.account["username"]) in data["text"]
- 
+  
       m["html"] = util.linkify(m["text"],
         ((util.PARSE_HASH, '#<a class="hash" href="%s#search?q=\\1">\\1</a>' % URL_PREFIX),
         (util.PARSE_NICK, '@<a class="nick" href="%s/\\1">\\1</a>' % URL_PREFIX)))
-
+  
       m["content"] = util.linkify(m["text"],
         ((util.PARSE_HASH, '#<a class="hash" href="gwibber:/tag?acct=%s&query=\\1">\\1</a>' % m["account"]),
         (util.PARSE_NICK, '@<a class="nick" href="gwibber:/user?acct=%s&name=\\1">\\1</a>' % m["account"])))
 
+      images = []
       if data.get("attachments", 0):
-        m["images"] = []
         for a in data["attachments"]:
           mime = a.get("mimetype", "")
           if mime and mime.startswith("image") and a.get("url", 0):
-            m["images"].append({"src": a["url"], "url": a["url"]})
+            images.append({"src": a["url"], "url": a["url"]})
+
+      images.extend(util.imgpreview(m["text"]))
+
+      if images:
+        m["images"] = images
     except:
       log.logger.error("%s failure - %s", PROTOCOL_INFO["name"], data)
-
+    
     return m
 
   def _message(self, data):
@@ -99,12 +105,20 @@ class Client:
     m["sender"]["image"] = user["profile_image_url"]
     m["sender"]["url"] = "/".join((URL_PREFIX, m["sender"]["nick"]))
     m["sender"]["is_me"] = m["sender"]["nick"] == self.account["username"]
-    m["url"] = "/".join((URL_PREFIX, "notice", m["id"]))
+    m["url"] = "/".join((URL_PREFIX, "notice", m["mid"]))
     return m
 
   def _private(self, data):
     m = self._message(data)
     m["private"] = True
+    m["recipient"] = {}
+    m["recipient"]["name"] = data["recipient"]["name"]
+    m["recipient"]["nick"] = data["recipient"]["screen_name"]
+    m["recipient"]["id"] = data["recipient"]["id"]
+    m["recipient"]["image"] = data["recipient"]["profile_image_url"]
+    m["recipient"]["location"] = data["recipient"]["location"]
+    m["recipient"]["url"] = "/".join((URL_PREFIX, m["recipient"]["nick"]))
+    m["recipient"]["is_me"] = m["recipient"]["nick"].lower() == self.account["username"].lower()
     return m
 
   def _result(self, data):
@@ -120,7 +134,7 @@ class Client:
     m["sender"]["id"] = data["from_user_id"]
     m["sender"]["image"] = data["profile_image_url"]
     m["sender"]["url"] = "/".join((URL_PREFIX, m["sender"]["nick"]))
-    m["url"] = "/".join((URL_PREFIX, "notice", str(m["id"])))
+    m["url"] = "/".join((URL_PREFIX, "notice", str(m["mid"])))
     return m
 
   def _get(self, path, parse="message", post=False, single=False, **args):
@@ -130,7 +144,7 @@ class Client:
 
     if isinstance(data, dict) and data.get("error", 0):
       if "authenticate" in data["error"]:
-        raise exceptions.GwibberProtocolError("auth", self.account["protocol"], self.account["username"], data["error"])
+        raise exceptions.GwibberServiceError("auth", self.account["service"], self.account["username"], data["error"])
     elif isinstance(data, dict) and data.get("error", 0):
       log.logger.error("%s failure - %s", PROTOCOL_INFO["name"], data["error"])
       return []
@@ -141,7 +155,7 @@ class Client:
     if single: return [getattr(self, "_%s" % parse)(data)]
     if parse: return [getattr(self, "_%s" % parse)(m) for m in data]
     else: return []
-      
+
   def _search(self, **args):
     data = network.Download("%s/api/search.json" % URL_PREFIX, util.compact(args))
     data = data.get_json()["results"]
@@ -172,17 +186,21 @@ class Client:
     return self._search(q="#%s" % query, count=count, since_id=since)
 
   def delete(self, message):
-    self._get("statuses/destroy/%s.json" % message["id"], None, post=True, do=1)
+    self._get("statuses/destroy/%s.json" % message["mid"], None, post=True, do=1)
     return []
 
   def like(self, message):
-    self._get("favorites/create/%s.json" % message["id"], None, post=True, do=1)
+    self._get("favorites/create/%s.json" % message["mid"], None, post=True, do=1)
     return []
 
   def send(self, message):
     return self._get("statuses/update.json", post=True, single=True,
-        status=message, source="Gwibber")
+        status=message, source="gwibber")
+
+  def send_private(self, message, private):
+    return self._get("direct_messages/new.json", "private", post=True, single=True,
+        text=message, screen_name=private["sender"]["nick"])
 
   def send_thread(self, message, target):
     return self._get("statuses/update.json", post=True, single=True,
-        status=message, source="gwibber", in_reply_to_status_id=target["id"])
+        status=message, source="gwibber", in_reply_to_status_id=target["mid"])
