@@ -5,9 +5,21 @@ import traceback
 from util import log
 from util import exceptions
 
+try:
+  import libproxy
+except:
+  libproxy = None
+
 class CurlDownloader:
-  def __init__(self, url, params=None, post=False, username=None, password=None):
+  def __init__(self, url, params=None, post=False, username=None, password=None, header=None, body=None):
     self.curl = pycurl.Curl()
+
+    if header:
+      self.curl.setopt(pycurl.HTTPHEADER, header)
+
+    if body:
+      self.curl.setopt(pycurl.POST, 1)
+      self.curl.setopt(pycurl.POSTFIELDS, body)
   
     if params:
       if post:
@@ -16,9 +28,10 @@ class CurlDownloader:
         url = "?".join((url, urllib.urlencode(params)))
     
     self.curl.setopt(pycurl.URL, str(url))
+    #log.logger.debug("URL: %s", str(url))
     
     if username and password:
-      self.curl.setopt(pycurl.USERPWD, "%s:%s" % (username, password))
+      self.curl.setopt(pycurl.USERPWD, "%s:%s" % (str(username), str(password)))
 
     self.curl.setopt(pycurl.FOLLOWLOCATION, 1)
     self.curl.setopt(pycurl.MAXREDIRS, 5)
@@ -28,12 +41,23 @@ class CurlDownloader:
     self.content = StringIO.StringIO()
     self.curl.setopt(pycurl.WRITEFUNCTION, self.content.write)
     
+    if libproxy:
+      pf = libproxy.ProxyFactory()
+      proxylist = pf.getProxies(str(url))
+
+      if proxylist:
+        proxy = proxylist[0]
+        if (proxy.find("@") != -1):
+            self.curl.setopt(pycurl.PROXYAUTH, ["CURLAUTH_ANY"])
+        if (proxy.find("direct://") != 0):
+            log.logger.debug("using proxy %s", proxy)
+            self.curl.setopt(pycurl.PROXY, proxy)
+
     try:
       self.curl.perform()
     except:
       traceback.print_exc()
       log.logger.error("Failed to communicate with %s", str(url))
-      #raise exceptions.GwibberProtocolError("network", str(url))
   
   def get_json(self):
     try:
