@@ -40,10 +40,10 @@ class Client:
     self.account = acct
 
   def _common(self, data):
-    m = {};
-    m["id"] = str(data["id"])
-    m["protocol"] = "qaiku"
-    m["account"] = self.account["_id"]
+    m = {}; 
+    m["mid"] = str(data["id"])
+    m["service"] = "qaiku"
+    m["account"] = self.account["id"]
     m["time"] = util.parsetime(data["created_at"])
     m["text"] = data["text"]
     m["to_me"] = ("@%s" % self.account["username"]) in data["text"]
@@ -85,7 +85,7 @@ class Client:
     m["sender"]["image"] = "/".join((URL_PREFIX, img)) if img[0] == "/" else img
     m["sender"]["url"] = user["url"]
     m["sender"]["is_me"] = m["sender"]["nick"] == self.account["username"]
-    m["url"] = "/".join((m["sender"]["url"], "show", m["id"]))
+    m["url"] = "/".join((m["sender"]["url"], "show", m["mid"]))
 
     return m
 
@@ -114,8 +114,6 @@ class Client:
     return self._get("statuses/update.json", post=True, single=True, status=message, source='gwibbernet')
 
   def send_thread(self, message, target):
-    recipient = target.get("reply_id", 0) or target.get("id", 0)
+    recipient = target.get("reply", {}).get("id", 0) or target.get("mid", 0)
     return self._get("statuses/update.json", post=True, single=True,
         status=message, in_reply_to_status_id=recipient, source='gwibbernet')
-
-
