@@ -34,14 +34,18 @@ PROTOCOL_INFO = {
     "retweet",
     "like",
     "send_thread",
+    "send_private",
     "user_messages",
     "sinceid",
+    "lists",
+    "list",
   ],
 
   "default_streams": [
     "receive",
     "responses",
     "private",
+    "lists",
   ],
 }
 
@@ -57,7 +61,7 @@ def unescape(s):
 class Client:
   def __init__(self, acct):
     if not acct.has_key("access_token") and not acct.has_key("secret_token"):
-      raise exceptions.GwibberProtocolError("keyring")
+      raise exceptions.GwibberServiceError("keyring")
     if acct.has_key("secret_token") and acct.has_key("password"): acct.pop("password")
     self.account = acct
     self.sigmethod = oauth.OAuthSignatureMethod_HMAC_SHA1()
@@ -67,9 +71,9 @@ class Client:
   def _common(self, data):
     m = {}; 
     try:
-      m["id"] = str(data["id"])
-      m["protocol"] = "twitter"
-      m["account"] = self.account["_id"]
+      m["mid"] = str(data["id"])
+      m["service"] = "twitter"
+      m["account"] = self.account["id"]
       m["time"] = util.parsetime(data["created_at"])
       m["text"] = unescape(data["text"])
       m["to_me"] = ("@%s" % self.account["username"]) in data["text"]
@@ -81,11 +85,27 @@ class Client:
       m["content"] = util.linkify(data["text"],
         ((util.PARSE_HASH, '#<a class="hash" href="gwibber:/tag?acct=%s&query=\\1">\\1</a>' % m["account"]),
         (util.PARSE_NICK, '@<a class="nick" href="gwibber:/user?acct=%s&name=\\1">\\1</a>' % m["account"])), escape=False)
+
+      images = util.imgpreview(m["text"])
+      if images:
+        m["images"] = images
     except: 
       log.logger.error("%s failure - %s", PROTOCOL_INFO["name"], data)
  
     return m
 
+  def _user(self, user):
+    return {
+        "name": user["name"],
+        "nick": user["screen_name"],
+        "id": user["id"],
+        "location": user["location"],
+        "followers": user.get("followers", None),
+        "image": user["profile_image_url"],
+        "url": "/".join((URL_PREFIX, user["screen_name"])),
+        "is_me": user["screen_name"] == self.account["username"],
+    }
+    
   def _message(self, data):
     if type(data) == type(None):
       return []
@@ -99,24 +119,24 @@ class Client:
       m["reply"]["nick"] = data["in_reply_to_screen_name"]
       m["reply"]["url"] = "/".join((URL_PREFIX, m["reply"]["nick"], "statuses", str(m["reply"]["id"])))
 
-    user = data["user"] if "user" in data else data["sender"]
-    
-    m["sender"] = {}
-    m["sender"]["name"] = user["name"]
-    m["sender"]["nick"] = user["screen_name"]
-    m["sender"]["id"] = user["id"]
-    m["sender"]["location"] = user["location"]
-    m["sender"]["followers"] = user["followers_count"]
-    m["sender"]["image"] = user["profile_image_url"]
-    m["sender"]["url"] = "/".join((URL_PREFIX, m["sender"]["nick"]))
-    m["sender"]["is_me"] = m["sender"]["nick"] == self.account["username"]
-    m["url"] = "/".join((m["sender"]["url"], "statuses", str(m["id"])))
+    m["sender"] = self._user(data["user"] if "user" in data else data["sender"])
+    m["url"] = "/".join((m["sender"]["url"], "statuses", str(m["mid"])))
 
     return m
 
   def _private(self, data):
     m = self._message(data)
     m["private"] = True
+    
+    m["recipient"] = {}
+    m["recipient"]["name"] = data["recipient"]["name"]
+    m["recipient"]["nick"] = data["recipient"]["screen_name"]
+    m["recipient"]["id"] = data["recipient"]["id"]
+    m["recipient"]["image"] = data["recipient"]["profile_image_url"]
+    m["recipient"]["location"] = data["recipient"]["location"]
+    m["recipient"]["url"] = "/".join((URL_PREFIX, m["recipient"]["nick"]))
+    m["recipient"]["is_me"] = m["recipient"]["nick"] == self.account["username"]
+
     return m
 
   def _result(self, data):
@@ -133,9 +153,31 @@ class Client:
     m["sender"]["image"] = data["profile_image_url"]
     m["sender"]["url"] = "/".join((URL_PREFIX, m["sender"]["nick"]))
     m["sender"]["is_me"] = m["sender"]["nick"] == self.account["username"]
-    m["url"] = "/".join((m["sender"]["url"], "statuses", str(m["id"])))
+    m["url"] = "/".join((m["sender"]["url"], "statuses", str(m["mid"])))
     return m
 
+  def _list(self, data):
+    return {
+        "mid": data["id"],
+        "service": "twitter",
+        "account": self.account["id"],
+        "time": 0,
+        "text": data["description"],
+        "html": data["description"],
+        "content": data["description"],
+        "url": "/".join((URL_PREFIX, data["uri"])),
+        "sender": self._user(data["user"]),
+        "name": data["name"],
+        "nick": data["slug"],
+        "key": data["slug"],
+        "full": data["full_name"],
+        "uri": data["uri"],
+        "mode": data["mode"],
+        "members": data["member_count"],
+        "followers": data["subscriber_count"],
+        "kind": "list",
+    }
+
   def _get(self, path, parse="message", post=False, single=False, **args):
     url = "/".join((API_PREFIX, path))
 
@@ -150,19 +192,24 @@ class Client:
 
     if isinstance(data, dict) and data.get("errors", 0):
       if "authenticate" in data["errors"][0]["message"]:
-        raise exceptions.GwibberProtocolError("auth",
-            self.account["protocol"], self.account["username"], data["errors"][0]["message"])
+        raise exceptions.GwibberServiceError("auth",
+            self.account["service"], self.account["username"], data["errors"][0]["message"])
       else:
         for error in data["errors"]:
           log.logger.info("Twitter failure - %s", error["message"])
         return []
     elif isinstance(data, dict) and data.get("error", 0):
       log.logger.error("%s failure - %s", PROTOCOL_INFO["name"], data["error"])
+      if "Incorrect signature" in data["error"]:
+        print data
+        raise exceptions.GwibberServiceError("keyring")
       return []
     elif isinstance(data, str):
       log.logger.error("%s unexpected result - %s", PROTOCOL_INFO["name"], data)
       return []
     
+    if parse == "list":
+      return [self._list(l) for l in data["lists"]]
     if single: return [getattr(self, "_%s" % parse)(data)]
     if parse: return [getattr(self, "_%s" % parse)(m) for m in data]
     else: return []
@@ -190,6 +237,14 @@ class Client:
   def public(self):
     return self._get("statuses/public_timeline.json")
 
+  def lists(self, **args):
+    following = self._get("%s/lists/subscriptions.json" % self.account["username"], "list")
+    lists = self._get("%s/lists.json" % self.account["username"], "list")
+    return following + lists
+
+  def list(self, user, id, count=util.COUNT, since=None):
+    return self._get("%s/lists/%s/statuses.json" % (user, id), per_page=count, since_id=since)
+
   def search(self, query, count=util.COUNT, since=None):
     return self._search(q=query, rpp=count, since_id=since)
 
@@ -197,15 +252,19 @@ class Client:
     return self._search(q="#%s" % query, count=count, since_id=since)
 
   def delete(self, message):
-    return self._get("statuses/destroy/%s.json" % message["id"], None, post=True, do=1)
+    return self._get("statuses/destroy/%s.json" % message["mid"], None, post=True, do=1)
 
   def like(self, message):
-    return self._get("favorites/create/%s.json" % message["id"], None, post=True, do=1)
+    return self._get("favorites/create/%s.json" % message["mid"], None, post=True, do=1)
 
   def send(self, message):
     return self._get("statuses/update.json", post=True, single=True,
         status=message)
   
+  def send_private(self, message, private):
+    return self._get("direct_messages/new.json", "private", post=True, single=True,
+        text=message, screen_name=private["sender"]["nick"])
+
   def send_thread(self, message, target):
     return self._get("statuses/update.json", post=True, single=True,
-        status=message, in_reply_to_status_id=target["id"])
+        status=message, in_reply_to_status_id=target["mid"])
