@@ -1,3 +1,6 @@
+import xdg.BaseDirectory
+from os.path import join, isdir, realpath
+from os import mkdir
 
 from os.path import join, isdir, realpath
 from os import mkdir
@@ -19,11 +22,18 @@ TWITTER_OAUTH_SECRET = "BqHlB8sMz5FhZmmFimwgiIdB0RiBr72Y0bio49IVJM"
 
 # Gwibber
 MAX_MESSAGE_LENGTH = 140
+MAX_MESSAGE_COUNT = 20000
+
+cache_dir = realpath(join(xdg.BaseDirectory.xdg_cache_home, "gwibber"))
+config_dir = realpath(join(xdg.BaseDirectory.xdg_config_home, "gwibber"))
+SQLITE_DB_FILENAME = "%s/gwibber.sqlite" % config_dir
+if not isdir(cache_dir): mkdir(cache_dir)
+if not isdir(config_dir): mkdir(config_dir)
 
 DEFAULT_SETTINGS = {
-  "interval": 5,
+  "interval": 15,
   "view": "SingleStreamUi",
-  "streams": [{"stream": "messages", "account": None}],
+  "streams": '[{"stream": "messages", "account": null}]',
   "show_notifications": True,
   "notify_mentions_only": True,
   "presence_check": True,
@@ -34,13 +44,13 @@ DEFAULT_SETTINGS = {
   "retweet_style": "recycle",
   "global_retweet": False,
   "theme": "default",
-}
-
-LOCAL_SETTINGS = {
   "window_size":  (500, 580),
   "window_position": (0, 24),
   "window_splitter": 450,
   "sidebar_splitter": 40,
+  "minimize_to_tray": False,
+  "hide_taskbar_entry": False,
+  "show_tray_icon": True,
 }
 
 RETWEET_FORMATS = {
@@ -53,7 +63,7 @@ RETWEET_FORMATS = {
   "service": "{R} @{nick}: {text}",
 }
 
-VERSION_NUMBER = "2.30.3"
+VERSION_NUMBER = "2.32.0.1"
 GCONF_CLIENT_DIR = "/apps/gwibber/client/"
 
 BUG_URL = "https://bugs.launchpad.net/gwibber/+filebug"
@@ -70,71 +80,6 @@ NM_STATE_CONNECTING = 2
 NM_STATE_CONNECTED = 3
 NM_STATE_DISCONNECTED = 4
 
-# Databases
-COUCH_DB_MESSAGES = "gwibber_messages"
-COUCH_DB_ACCOUNTS = "gwibber_accounts"
-COUCH_DB_SETTINGS = "gwibber_preferences"
-
-# Record types
-COUCH_TYPE_ACCOUNT = "http://gwibber.com/couch/account"
-COUCH_TYPE_MESSAGE = "http://gwibber.com/couch/message"
-COUCH_TYPE_STREAM = "http://gwibber.com/couch/stream"
-COUCH_TYPE_SEARCH = "http://gwibber.com/couch/search"
-COUCH_TYPE_CONFIG = "http://gwibber.com/couch/settings"
-
-# Records
-COUCH_RECORD_SETTINGS = "settings"
-
-# Views
-COUCH_VIEW_MESSAGES = {
-  "message": {
-    "map": "function(doc) { emit([doc.id, doc.account, doc.operation], null); }"
-  },
-  "home": {
-    "map": "function(doc) { if(!doc.transient) emit([doc.time], null); }"
-  },
-  "maxid": {
-    "map": "function(doc) { emit([doc.transient ? doc.transient : doc.account, doc.operation], doc.id); }",
-    "reduce": "function(key, value, rereduce) { return Math.max.apply(Math, value); }"
-  },
-  "sender": {
-    "map": "function(doc) { emit([doc.sender.nick, doc.account], null); }"
-  },
-  "account_stream_time": {
-    "map": "function(doc) { emit([doc.account, doc.stream, doc.time], null); }"
-  },
-  "account_time": {
-    "map": "function(doc) { if(!doc.transient) emit([doc.account, doc.time], null); }"
-  },
-  "stream_time": {
-    "map": "function(doc) { if(!doc.transient) emit([doc.stream, doc.time], null); }"
-  },
-  "transient_time": {
-    "map": "function(doc) { if(doc.transient) emit([doc.transient, doc.time], null); }"
-  },
-  "search_transient_time": {
-    "map": "function(doc) { if(doc.stream == \"search\") emit([doc.transient, doc.time], null); }"
-  },
-  "search_time": {
-    "map": "function(doc) { if(doc.stream == \"search\") emit(doc.time, null); }"
-  },
-  "user_protocol_time": {
-    "map": """function(doc) {
-      if(doc.transient) emit([doc.sender.nick, doc.protocol, doc.time], null);
-      if(doc.sender.is_me && doc.reply) emit([doc.reply.nick, doc.protocol, doc.time], null);
-    }"""
-  },
-  "mine_account": {
-    "map": "function(doc) { if(doc.sender.is_me) emit([doc.account, doc.time], null); }",
-  },
-  "mine": {
-    "map": "function(doc) { if(doc.sender.is_me) emit([doc.time], null); }",
-  },
-  "max_message_time": {
-      "map": "function(doc) { emit([doc.account, doc.operation], doc.time); }",
-      "reduce": "function(key, value, rereduce) { return Math.max.apply(Math, value); }"
-  },
-}
 
 GWIBBER_OPERATIONS = """
 {
@@ -186,6 +131,31 @@ GWIBBER_OPERATIONS = """
     "transient": false
   },
 
+  "lists": {
+    "account_tree": true,
+    "dynamic": false,
+    "enabled": "receive",
+    "first_only": false,
+    "function": null,
+    "return_value": true,
+    "search": false,
+    "stream": "lists",
+    "transient": true,
+    "interval": 20
+  },
+
+  "list": {
+    "account_tree": true,
+    "dynamic": false,
+    "enabled": "receive",
+    "first_only": false,
+    "function": null,
+    "return_value": true,
+    "search": false,
+    "stream": "list",
+    "transient": true
+  },
+
   "private": {
     "account_tree": true,
     "dynamic": false,
@@ -317,6 +287,18 @@ GWIBBER_OPERATIONS = """
     "stream": null,
     "transient": false
   },
+
+  "send_private": {
+    "account_tree": false,
+    "dynamic": false,
+    "enabled": "send",
+    "first_only": false,
+    "function": null,
+    "return_value": false,
+    "search": false,
+    "stream": null,
+    "transient": false
+  },
   
   "tag": {
     "account_tree": false,
