@@ -12,14 +12,14 @@
 #     * Redistributions in binary form must reproduce the above copyright
 #       notice, this list of conditions and the following disclaimer in the
 #       documentation and/or other materials provided with the distribution.
-#     * Neither the name of the <organization> nor the
-#       names of its contributors may be used to endorse or promote products
-#       derived from this software without specific prior written permission.
+#     * Neither the name of the author nor the names of its contributors may
+#       be used to endorse or promote products derived from this software
+#       without specific prior written permission.
 #
-# THIS SOFTWARE IS PROVIDED BY <copyright holder> ``AS IS'' AND ANY
+# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS``AS IS'' AND ANY
 # EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-# DISCLAIMED. IN NO EVENT SHALL <copyright holder> BE LIABLE FOR ANY
+# DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 # DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 # (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 # LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
@@ -46,32 +46,75 @@ apt-get install python-simplejson on a Debian-like system.
 
 import sys
 import time
+import struct
 import urllib
 import urllib2
 import httplib
+try:
+    import hashlib
+except ImportError:
+    import md5 as hashlib
+import binascii
+import urlparse
 import mimetypes
 
 import pycurl
 import StringIO
 
-# try to use simplejson first, otherwise fallback to XML
 try:
-    import simplejson
-    RESPONSE_FORMAT = 'JSON'
+    import libproxy
+except:
+    libproxy = None
+
+# try to use simplejson first, otherwise fallback to XML
+RESPONSE_FORMAT = 'JSON'
+try:
+    import json as simplejson
 except ImportError:
     try:
-        from django.utils import simplejson
-        RESPONSE_FORMAT = 'JSON'
+        import simplejson
     except ImportError:
-        from xml.dom import minidom
-        RESPONSE_FORMAT = 'XML'
+        try:
+            from django.utils import simplejson
+        except ImportError:
+            try:
+                import jsonlib as simplejson
+                simplejson.loads
+            except (ImportError, AttributeError):
+                from xml.dom import minidom
+                RESPONSE_FORMAT = 'XML'
+
+# support Google App Engine.  GAE does not have a working urllib.urlopen.
+try:
+    from google.appengine.api import urlfetch
+
+    def urlread(url, data=None, headers=None):
+        if data is not None:
+            if headers is None:
+                headers = {"Content-type": "application/x-www-form-urlencoded"}
+            method = urlfetch.POST
+        else:
+            if headers is None:
+                headers = {}
+            method = urlfetch.GET
+
+        result = urlfetch.fetch(url, method=method,
+                                payload=data, headers=headers)
+
+        if result.status_code == 200:
+            return result.content
+        else:
+            raise urllib2.URLError("fetch error url=%s, code=%d" % (url, result.status_code))
+
+except ImportError:
+    def urlread(url, data=None):
+        res = urllib2.urlopen(url, data=data)
+        return res.read()
 
 __all__ = ['Facebook']
 
 VERSION = '0.1'
 
-# REST URLs
-# Change these to /bestserver.php to use the bestserver.
 FACEBOOK_URL = 'http://api.facebook.com/restserver.php'
 FACEBOOK_SECURE_URL = 'https://api.facebook.com/restserver.php'
 
@@ -79,6 +122,28 @@ class json(object): pass
 
 # simple IDL for the Facebook API
 METHODS = {
+    'application': {
+        'getPublicInfo': [
+            ('application_id', int, ['optional']),
+            ('application_api_key', str, ['optional']),
+            ('application_canvas_name', str,['optional']),
+        ],
+    },
+
+    # admin methods
+    'admin': {
+        'getAllocation': [
+            ('integration_point_name', str, []),
+        ],
+    },
+
+    # auth methods
+    'auth': {
+        'revokeAuthorization': [
+            ('uid', int, ['optional']),
+        ],
+    },
+
     # feed methods
     'feed': {
         'publishStoryToUser': [
@@ -110,15 +175,11 @@ METHODS = {
         ],
 
         'publishTemplatizedAction': [
-            # facebook expects title_data and body_data to be JSON
-            # simplejson.dumps({'place':'Florida'}) would do fine
-            # actor_id is now deprecated, use page_actor_id instead
-            ('actor_id', int, []),
-            ('page_actor_id', int, []),
             ('title_template', str, []),
-            ('title_data', str, ['optional']),
+            ('page_actor_id', int, ['optional']),
+            ('title_data', json, ['optional']),
             ('body_template', str, ['optional']),
-            ('body_data', str, ['optional']),
+            ('body_data', json, ['optional']),
             ('body_general', str, ['optional']),
             ('image_1', str, ['optional']),
             ('image_1_link', str, ['optional']),
@@ -130,6 +191,31 @@ METHODS = {
             ('image_4_link', str, ['optional']),
             ('target_ids', list, ['optional']),
         ],
+
+        'registerTemplateBundle': [
+            ('one_line_story_templates', json, []),
+            ('short_story_templates', json, ['optional']),
+            ('full_story_template', json, ['optional']),
+            ('action_links', json, ['optional']),
+        ],
+
+        'deactivateTemplateBundleByID': [
+            ('template_bundle_id', int, []),
+        ],
+
+        'getRegisteredTemplateBundles': [],
+
+        'getRegisteredTemplateBundleByID': [
+            ('template_bundle_id', str, []),
+        ],
+
+        'publishUserAction': [
+            ('template_bundle_id', int, []),
+            ('template_data', json, ['optional']),
+            ('target_ids', list, ['optional']),
+            ('body_general', str, ['optional']),
+            ('story_size', int, ['optional']),
+        ],
     },
 
     # fql methods
@@ -146,7 +232,11 @@ METHODS = {
             ('uids2', list, []),
         ],
 
-        'get': [],
+        'get': [
+            ('flid', int, ['optional']),
+        ],
+
+        'getLists': [],
 
         'getAppUsers': [],
     },
@@ -159,6 +249,7 @@ METHODS = {
             ('to_ids', list, []),
             ('notification', str, []),
             ('email', str, ['optional']),
+            ('type', str, ['optional']),
         ],
 
         'sendRequest': [
@@ -185,10 +276,32 @@ METHODS = {
             ('profile', str, ['optional']),
             ('profile_action', str, ['optional']),
             ('mobile_fbml', str, ['optional']),
+            ('profile_main', str, ['optional']),
         ],
 
         'getFBML': [
             ('uid', int, ['optional']),
+            ('type', int, ['optional']),
+        ],
+
+        'setInfo': [
+            ('title', str, []),
+            ('type', int, []),
+            ('info_fields', json, []),
+            ('uid', int, []),
+        ],
+
+        'getInfo': [
+            ('uid', int, []),
+        ],
+
+        'setInfoOptions': [
+            ('field', str, []),
+            ('options', json, []),
+        ],
+
+        'getInfoOptions': [
+            ('field', str, []),
         ],
     },
 
@@ -199,18 +312,25 @@ METHODS = {
             ('fields', list, [('default', ['name'])]),
         ],
 
+        'getStandardInfo': [
+            ('uids', list, []),
+            ('fields', list, [('default', ['uid'])]),
+        ],
+
         'getLoggedInUser': [],
 
         'isAppAdded': [],
 
         'hasAppPermission': [
             ('ext_perm', str, []),
+            ('uid', int, ['optional']),
         ],
 
         'setStatus': [
             ('status', str, []),
-            ('status_includes_verb', bool, []),
-            #('clear', bool, []),
+            ('clear', bool, []),
+            ('status_includes_verb', bool, ['optional']),
+            ('uid', int, ['optional']),
         ],
     },
 
@@ -227,6 +347,10 @@ METHODS = {
         'getMembers': [
             ('eid', int, []),
         ],
+
+        'create': [
+            ('event_info', json, []),
+        ],
     },
 
     # update methods
@@ -282,6 +406,7 @@ METHODS = {
     # pages methods
     'pages': {
         'getInfo': [
+            ('fields', list, [('default', ['page_id', 'name'])]),
             ('page_ids', list, ['optional']),
             ('uid', int, ['optional']),
         ],
@@ -333,6 +458,18 @@ METHODS = {
         ],
     },
 
+    # status methods
+    'status': {
+        'get': [
+            ('uid', int, ['optional']),
+            ('limit', int, ['optional']),
+        ],
+        'set': [
+            ('status', str, ['optional']),
+            ('uid', int, ['optional']),
+        ],
+    },
+
     # fbml methods
     'fbml': {
         'refreshImgSrc': [
@@ -349,10 +486,24 @@ METHODS = {
         ],
     },
 
+    # SMS Methods
+    'sms' : {
+        'canSend' : [
+            ('uid', int, []),
+        ],
+
+        'send' : [
+            ('uid', int, []),
+            ('message', str, []),
+            ('session_id', int, []),
+            ('req_session', bool, []),
+        ],
+    },
+
     'data': {
         'getCookies': [
             ('uid', int, []),
-            ('string', str, []),
+            ('string', str, ['optional']),
         ],
 
         'setCookie': [
@@ -364,6 +515,21 @@ METHODS = {
         ],
     },
 
+    # connect methods
+    'connect': {
+        'registerUsers': [
+            ('accounts', json, []),
+        ],
+
+        'unregisterUsers': [
+            ('email_hashes', json, []),
+        ],
+
+        'getUnconnectedFriendsCount': [
+        ],
+    },
+
+    #stream methods (beta)
     'stream' : {
         'addComment' : [
             ('post_id', int, []),
@@ -375,7 +541,7 @@ METHODS = {
             ('uid', int, ['optional']),
             ('post_id', int, ['optional']),
         ],
-            
+
         'get' : [
             ('viewer_id', int, ['optional']),
             ('source_ids', list, ['optional']),
@@ -384,33 +550,33 @@ METHODS = {
             ('limit', int, ['optional']),
             ('filter_key', str, ['optional']),
         ],
-            
+
         'getComments' : [
             ('post_id', int, []),
         ],
-            
+
         'getFilters' : [
             ('uid', int, ['optional']),
         ],
-            
+
         'publish' : [
             ('message', str, ['optional']),
-            ('attachment', object, ['optional']),
-            ('action_links', list, ['optional']),
+            ('attachment', json, ['optional']),
+            ('action_links', json, ['optional']),
             ('target_id', str, ['optional']),
             ('uid', str, ['optional']),
-        ],    
-        
+        ],
+
         'remove' : [
             ('post_id', int, []),
             ('uid', int, ['optional']),
         ],
-        
+
         'removeComment' : [
             ('comment_id', int, []),
             ('uid', int, ['optional']),
         ],
-        
+
         'removeLike' : [
             ('uid', int, ['optional']),
             ('post_id', int, ['optional']),
@@ -418,7 +584,6 @@ METHODS = {
     }
 }
 
-
 class Proxy(object):
     """Represents a "namespace" of Facebook API calls."""
 
@@ -426,7 +591,11 @@ class Proxy(object):
         self._client = client
         self._name = name
 
-    def __call__(self, method, args=None, add_session_args=True):
+    def __call__(self, method=None, args=None, add_session_args=True):
+        # for Django templates
+        if method is None:
+            return self
+
         if add_session_args:
             self._client._add_session_args(args)
 
@@ -453,6 +622,10 @@ def __generate_proxies():
                         else:
                             param = '%s=%s' % (param_name, repr(option[1]))
 
+                if param_type == json:
+                    # we only jsonify the argument if it's a list or a dict, for compatibility
+                    body.append('if isinstance(%s, list) or isinstance(%s, dict): %s = simplejson.dumps(%s)' % ((param_name,) * 4))
+
                 if 'optional' in param_options:
                     param = '%s=None' % param_name
                     body.append('if %s is not None: args[\'%s\'] = %s' % (param_name, param_name, param_name))
@@ -492,7 +665,7 @@ class FacebookError(Exception):
         return 'Error %s: %s' % (self.code, self.msg)
 
 
-class AuthProxy(Proxy):
+class AuthProxy(AuthProxy):
     """Special proxy for facebook.auth."""
 
     def getSession(self):
@@ -519,17 +692,17 @@ class AuthProxy(Proxy):
 class FriendsProxy(FriendsProxy):
     """Special proxy for facebook.friends."""
 
-    def get(self):
+    def get(self, **kwargs):
         """Facebook API call. See http://developers.facebook.com/documentation.php?v=1.0&method=friends.get"""
-        if self._client._friends:
+        if not kwargs.get('flid') and self._client._friends:
             return self._client._friends
-        return super(FriendsProxy, self).get()
+        return super(FriendsProxy, self).get(**kwargs)
 
 
 class PhotosProxy(PhotosProxy):
     """Special proxy for facebook.photos."""
 
-    def upload(self, image, aid=None, caption=None, size=(604, 1024)):
+    def upload(self, image, aid=None, caption=None, size=(604, 1024), filename=None, callback=None):
         """Facebook API call. See http://developers.facebook.com/documentation.php?v=1.0&method=photos.upload
 
         size -- an optional size (width, height) to resize the image to before uploading. Resizes by default
@@ -550,33 +723,72 @@ class PhotosProxy(PhotosProxy):
         except ImportError:
             import StringIO
 
-        try:
-            import Image
-        except ImportError:
-            data = StringIO.StringIO(open(image, 'rb').read())
+        # check for a filename specified...if the user is passing binary data in
+        # image then a filename will be specified
+        if filename is None:
+            try:
+                import Image
+            except ImportError:
+                data = StringIO.StringIO(open(image, 'rb').read())
+            else:
+                img = Image.open(image)
+                if size:
+                    img.thumbnail(size, Image.ANTIALIAS)
+                data = StringIO.StringIO()
+                img.save(data, img.format)
         else:
-            img = Image.open(image)
-            if size:
-                img.thumbnail(size, Image.ANTIALIAS)
-            data = StringIO.StringIO()
-            img.save(data, img.format)
+            # there was a filename specified, which indicates that image was not
+            # the path to an image file but rather the binary data of a file
+            data = StringIO.StringIO(image)
+            image = filename
 
-        content_type, body = self.__encode_multipart_formdata(list(args.items()), [(image, data)])
-        h = httplib.HTTP('api.facebook.com')
-        h.putrequest('POST', '/restserver.php')
-        h.putheader('Content-Type', content_type)
-        h.putheader('Content-Length', str(len(body)))
-        h.putheader('MIME-Version', '1.0')
-        h.putheader('User-Agent', 'PyFacebook Client Library')
-        h.endheaders()
-        h.send(body)
+        content_type, body = self.__encode_multipart_formdata(list(args.iteritems()), [(image, data)])
+        urlinfo = urlparse.urlsplit(self._client.facebook_url)
+        try:
+            content_length = len(body)
+            chunk_size = 4096
 
-        reply = h.getreply()
+            h = httplib.HTTPConnection(urlinfo[1])
+            h.putrequest('POST', urlinfo[2])
+            h.putheader('Content-Type', content_type)
+            h.putheader('Content-Length', str(content_length))
+            h.putheader('MIME-Version', '1.0')
+            h.putheader('User-Agent', 'PyFacebook Client Library')
+            h.endheaders()
 
-        if reply[0] != 200:
-            raise Exception('Error uploading photo: Facebook returned HTTP %s (%s)' % (reply[0], reply[1]))
+            if callback:
+                count = 0
+                while len(body) > 0:
+                    if len(body) < chunk_size:
+                        data = body
+                        body = ''
+                    else:
+                        data = body[0:chunk_size]
+                        body = body[chunk_size:]
 
-        response = h.file.read()
+                    h.send(data)
+                    count += 1
+                    callback(count, chunk_size, content_length)
+            else:
+                h.send(body)
+
+            response = h.getresponse()
+
+            if response.status != 200:
+                raise Exception('Error uploading photo: Facebook returned HTTP %s (%s)' % (response.status, response.reason))
+            response = response.read()
+        except:
+            # sending the photo failed, perhaps we are using GAE
+            try:
+                from google.appengine.api import urlfetch
+
+                try:
+                    response = urlread(url=self._client.facebook_url,data=body,headers={'POST':urlinfo[2],'Content-Type':content_type,'MIME-Version':'1.0'})
+                except urllib2.URLError:
+                    raise Exception('Error uploading photo: Facebook returned %s' % (response))
+            except ImportError:
+                # could not import from google.appengine.api, so we are not running in GAE
+                raise Exception('Error uploading photo.')
 
         return self._client._parse_response(response, 'facebook.photos.upload')
 
@@ -640,15 +852,40 @@ class Facebook(object):
         True if this is a desktop app, False otherwise. Used for determining how to
         authenticate.
 
+    ext_perms
+        Any extended permissions that the user has granted to your application.
+        This parameter is set only if the user has granted any.
+
+    facebook_url
+        The url to use for Facebook requests.
+
+    facebook_secure_url
+        The url to use for secure Facebook requests.
+
     in_canvas
         True if the current request is for a canvas page.
 
+    in_iframe
+        True if the current request is for an HTML page to embed in Facebook inside an iframe.
+
+    is_session_from_cookie
+        True if the current request session comes from a session cookie.
+
+    in_profile_tab
+        True if the current request is for a user's tab for your application.
+
     internal
         True if this Facebook object is for an internal application (one that can be added on Facebook)
 
+    locale
+        The user's locale. Default: 'en_US'
+
     page_id
         Set to the page_id of the current page (if any)
 
+    profile_update_time
+        The time when this user's profile was last updated. This is a UNIX timestamp. Default: None if unknown.
+
     secret
         Secret that is used after getSession for desktop apps.
 
@@ -670,7 +907,7 @@ class Facebook(object):
 
     """
 
-    def __init__(self, api_key, secret_key, auth_token=None, app_name=None, callback_path=None, internal=None):
+    def __init__(self, api_key, secret_key, auth_token=None, app_name=None, callback_path=None, internal=None, proxy=None, facebook_url=None, facebook_secure_url=None):
         """
         Initializes a new Facebook object which provides wrappers for the Facebook API.
 
@@ -696,27 +933,36 @@ class Facebook(object):
         self.uid = None
         self.page_id = None
         self.in_canvas = False
+        self.in_iframe = False
+        self.is_session_from_cookie = False
+        self.in_profile_tab = False
         self.added = False
         self.app_name = app_name
         self.callback_path = callback_path
         self.internal = internal
         self._friends = None
+        self.locale = 'en_US'
+        self.profile_update_time = None
+        self.ext_perms = None
+        self.proxy = proxy
+        if facebook_url is None:
+            self.facebook_url = FACEBOOK_URL
+        else:
+            self.facebook_url = facebook_url
+        if facebook_secure_url is None:
+            self.facebook_secure_url = FACEBOOK_SECURE_URL
+        else:
+            self.facebook_secure_url = facebook_secure_url
 
         for namespace in METHODS:
             self.__dict__[namespace] = eval('%sProxy(self, \'%s\')' % (namespace.title(), 'facebook.%s' % namespace))
 
-        self.auth = AuthProxy(self, 'facebook.auth')
-
 
     def _hash_args(self, args, secret=None):
         """Hashes arguments by joining key=value pairs, appending a secret, and then taking the MD5 hex digest."""
-        try:
-            import hashlib
-            hasher = hashlib.md5(''.join(['%s=%s' % (x, args[x]) for x in sorted(args.keys())]))
-        except ImportError:
-            import md5
-            hasher = md5.new(''.join(['%s=%s' % (x, args[x]) for x in sorted(args.keys())]))
-
+        # @author: houyr
+        # fix for UnicodeEncodeError
+        hasher = hashlib.md5(''.join(['%s=%s' % (isinstance(x, unicode) and x.encode("utf-8") or x, isinstance(args[x], unicode) and args[x].encode("utf-8") or args[x]) for x in sorted(args.keys())]))
         if secret:
             hasher.update(secret)
         elif self.secret:
@@ -738,7 +984,7 @@ class Facebook(object):
             node.hasAttribute('list') and \
             node.getAttribute('list')=="true":
             return self._parse_response_list(node)
-        elif len([x for x in node.childNodes if x.nodeType == x.ELEMENT_NODE]) > 0:
+        elif len(filter(lambda x: x.nodeType == x.ELEMENT_NODE, node.childNodes)) > 0:
             return self._parse_response_dict(node)
         else:
             return ''.join(node.data for node in node.childNodes if node.nodeType == node.TEXT_NODE)
@@ -747,7 +993,7 @@ class Facebook(object):
     def _parse_response_dict(self, node):
         """Parses an XML dictionary response node from Facebook."""
         result = {}
-        for item in [x for x in node.childNodes if x.nodeType == x.ELEMENT_NODE]:
+        for item in filter(lambda x: x.nodeType == x.ELEMENT_NODE, node.childNodes):
             result[item.nodeName] = self._parse_response_item(item)
         if node.nodeType == node.ELEMENT_NODE and node.hasAttributes():
             if node.hasAttribute('id'):
@@ -758,14 +1004,14 @@ class Facebook(object):
     def _parse_response_list(self, node):
         """Parses an XML list response node from Facebook."""
         result = []
-        for item in [x for x in node.childNodes if x.nodeType == x.ELEMENT_NODE]:
+        for item in filter(lambda x: x.nodeType == x.ELEMENT_NODE, node.childNodes):
             result.append(self._parse_response_item(item))
         return result
 
 
     def _check_error(self, response):
         """Checks if the given Facebook response is an error, and then raises the appropriate exception."""
-        if isinstance(response, dict) and 'error_code' in response:
+        if type(response) is dict and response.has_key('error_code'):
             raise FacebookError(response['error_code'], response['error_msg'], response['request_args'])
 
 
@@ -774,11 +1020,13 @@ class Facebook(object):
         if args is None:
             args = {}
 
-        for arg in list(args.items()):
-            if isinstance(arg[1], list):
+        for arg in args.items():
+            if type(arg[1]) == list:
                 args[arg[0]] = ','.join(str(a) for a in arg[1])
-            elif isinstance(arg[1], unicode):
+            elif type(arg[1]) == unicode:
                 args[arg[0]] = arg[1].encode("UTF-8")
+            elif type(arg[1]) == bool:
+                args[arg[0]] = str(arg[1]).lower()
 
         args['method'] = method
         args['api_key'] = self.api_key
@@ -795,7 +1043,9 @@ class Facebook(object):
             args = {}
 
         if not self.session_key:
-            raise RuntimeError('Session key not set. Make sure auth.getSession has been called.')
+            return args
+            #some calls don't need a session anymore. this might be better done in the markup
+            #raise RuntimeError('Session key not set. Make sure auth.getSession has been called.')
 
         args['session_key'] = self.session_key
         args['call_id'] = str(int(time.time() * 1000))
@@ -827,8 +1077,40 @@ class Facebook(object):
         return result
 
 
-    def __call__(self, method, args=None, secure=True):
+    def hash_email(self, email):
+        """
+        Hash an email address in a format suitable for Facebook Connect.
+
+        """
+        email = email.lower().strip()
+        return "%s_%s" % (
+            struct.unpack("I", struct.pack("i", binascii.crc32(email)))[0],
+            hashlib.md5(email).hexdigest(),
+        )
+
+
+    def unicode_urlencode(self, params):
+        """
+        @author: houyr
+        A unicode aware version of urllib.urlencode.
+        """
+        if isinstance(params, dict):
+            params = params.items()
+        return urllib.urlencode([(k, isinstance(v, unicode) and v.encode('utf-8') or v)
+                          for k, v in params])
+
+
+    def __call__(self, method=None, args=None, secure=False):
         """Make a call to Facebook's REST server."""
+        # for Django templates, if this object is called without any arguments
+        # return the object itself
+        if method is None:
+            return self
+
+        # __init__ hard-codes into en_US
+        if args is not None and not args.has_key('locale'):
+            args['locale'] = self.locale
+
         post_data = self._build_post_args(method, args)
 
         c = pycurl.Curl()
@@ -836,6 +1118,16 @@ class Facebook(object):
         c.setopt(pycurl.HTTPHEADER, ["Accept:"])
         c.setopt(c.HTTPPOST, [(x,str(y)) for x,y in post_data.items()])
 
+        if libproxy:
+            pf = libproxy.ProxyFactory()
+            proxylist = pf.getProxies(FACEBOOK_SECURE_URL if secure else FACEBOOK_URL)
+            if proxylist:
+                proxy = proxylist[0]
+                if (proxy.find("@") != -1):
+                    self.curl.setopt(pycurl.PROXYAUTH, ["CURLAUTH_ANY"])
+                if (proxy.find("direct://") != 0):
+                    self.curl.setopt(pycurl.PROXY, proxy)
+
         b = StringIO.StringIO()
         c.setopt(pycurl.WRITEFUNCTION, b.write)
         c.setopt(pycurl.FOLLOWLOCATION, 1)
@@ -861,7 +1153,7 @@ class Facebook(object):
     def get_app_url(self, path=''):
         """
         Returns the URL for this app's canvas page, according to app_name.
-        
+
         """
         return 'http://apps.facebook.com/%s/%s' % (self.app_name, path)
 
@@ -881,7 +1173,8 @@ class Facebook(object):
 
     def get_authorize_url(self, next=None, next_cancel=None):
         """
-        Returns the URL that the user should be redirected to in order to authorize certain actions for application.
+        Returns the URL that the user should be redirected to in order to
+        authorize certain actions for application.
 
         """
         args = {'api_key': self.api_key, 'v': '1.0'}
@@ -906,7 +1199,7 @@ class Facebook(object):
 
         if next is not None:
             args['next'] = next
-			
+
         if canvas is True:
             args['canvas'] = 1
 
@@ -925,6 +1218,31 @@ class Facebook(object):
         webbrowser.open(self.get_login_url(popup=popup))
 
 
+    def get_ext_perm_url(self, ext_perm, next=None, popup=False):
+        """
+        Returns the URL that the user should be redirected to in order to grant an extended permission.
+
+        ext_perm -- the name of the extended permission to request
+        next     -- the URL that Facebook should redirect to after login
+
+        """
+        args = {'ext_perm': ext_perm, 'api_key': self.api_key, 'v': '1.0'}
+
+        if next is not None:
+            args['next'] = next
+
+        if popup is True:
+            args['popup'] = 1
+
+        return self.get_url('authorize', **args)
+
+
+    def request_extended_permission(self, ext_perm, popup=False):
+        """Open a web browser telling the user to grant an extended permission."""
+        import webbrowser
+        webbrowser.open(self.get_ext_perm_url(ext_perm, popup=popup))
+
+
     def check_session(self, request):
         """
         Checks the given Django HttpRequest for Facebook parameters such as
@@ -939,6 +1257,7 @@ class Facebook(object):
         if self.session_key and (self.uid or self.page_id):
             return True
 
+
         if request.method == 'POST':
             params = self.validate_signature(request.POST)
         else:
@@ -961,18 +1280,47 @@ class Facebook(object):
 
             params = self.validate_signature(request.GET)
 
+        if not params:
+            # first check if we are in django - to check cookies
+            if hasattr(request, 'COOKIES'):
+                params = self.validate_cookie_signature(request.COOKIES)
+                self.is_session_from_cookie = True
+            else:
+                # if not, then we might be on GoogleAppEngine, check their request object cookies
+                if hasattr(request,'cookies'):
+                    params = self.validate_cookie_signature(request.cookies)
+                    self.is_session_from_cookie = True
+
         if not params:
             return False
 
         if params.get('in_canvas') == '1':
             self.in_canvas = True
 
+        if params.get('in_iframe') == '1':
+            self.in_iframe = True
+
+        if params.get('in_profile_tab') == '1':
+            self.in_profile_tab = True
+
         if params.get('added') == '1':
             self.added = True
 
         if params.get('expires'):
             self.session_key_expires = int(params['expires'])
 
+        if 'locale' in params:
+            self.locale = params['locale']
+
+        if 'profile_update_time' in params:
+            try:
+                self.profile_update_time = int(params['profile_update_time'])
+            except ValueError:
+                pass
+
+        if 'ext_perms' in params:
+            self.ext_perms = params['ext_perms']
+
         if 'friends' in params:
             if params['friends']:
                 self._friends = params['friends'].split(',')
@@ -987,6 +1335,16 @@ class Facebook(object):
                 self.page_id = params['page_id']
             else:
                 return False
+        elif 'profile_session_key' in params:
+            self.session_key = params['profile_session_key']
+            if 'profile_user' in params:
+                self.uid = params['profile_user']
+            else:
+                return False
+        elif 'canvas_user' in params:
+            self.uid = params['canvas_user']
+        elif 'uninstall' in params:
+            self.uid = params['user']
         else:
             return False
 
@@ -1008,7 +1366,7 @@ class Facebook(object):
         if timeout and '%s_time' % prefix in post and time.time() - float(post['%s_time' % prefix]) > timeout:
             return None
 
-        args = dict([(key[len(prefix + '_'):], value) for key, value in list(args.items()) if key.startswith(prefix)])
+        args = dict([(key[len(prefix + '_'):], value) for key, value in args.items() if key.startswith(prefix)])
 
         hash = self._hash_args(args)
 
@@ -1017,6 +1375,38 @@ class Facebook(object):
         else:
             return None
 
+    def validate_cookie_signature(self, cookies):
+        """
+        Validate parameters passed by cookies, namely facebookconnect or js api.
+        """
+
+        api_key = self.api_key
+        if api_key not in cookies:
+            return None
+
+        prefix = api_key + "_"
+       
+        params = {} 
+        vals = ''
+        for k in sorted(cookies):
+            if k.startswith(prefix):
+                key = k.replace(prefix,"")
+                value = cookies[k]
+                params[key] = value
+                vals += '%s=%s' % (key, value)
+                
+        hasher = hashlib.md5(vals)
+
+        hasher.update(self.secret_key)
+        digest = hasher.hexdigest()
+        if digest == cookies[api_key]:
+            params['is_session_from_cookie'] = True
+            return params
+        else:
+            return False
+
+
+
 
 if __name__ == '__main__':
     # sample desktop application
@@ -1033,24 +1423,24 @@ if __name__ == '__main__':
     facebook.login()
 
     # Login to the window, then press enter
-    print('After logging in, press enter...')
+    print 'After logging in, press enter...'
     raw_input()
 
     facebook.auth.getSession()
-    print('Session Key:   ', facebook.session_key)
-    print('Your UID:      ', facebook.uid)
+    print 'Session Key:   ', facebook.session_key
+    print 'Your UID:      ', facebook.uid
 
     info = facebook.users.getInfo([facebook.uid], ['name', 'birthday', 'affiliations', 'sex'])[0]
 
-    print('Your Name:     ', info['name'])
-    print('Your Birthday: ', info['birthday'])
-    print('Your Gender:   ', info['sex'])
+    print 'Your Name:     ', info['name']
+    print 'Your Birthday: ', info['birthday']
+    print 'Your Gender:   ', info['sex']
 
     friends = facebook.friends.get()
     friends = facebook.users.getInfo(friends[0:5], ['name', 'birthday', 'relationship_status'])
 
     for friend in friends:
-        print(friend['name'], 'has a birthday on', friend['birthday'], 'and is', friend['relationship_status'])
+        print friend['name'], 'has a birthday on', friend['birthday'], 'and is', friend['relationship_status']
 
     arefriends = facebook.friends.areFriends([friends[0]['uid']], [friends[1]['uid']])
 
