@@ -1,12 +1,12 @@
 from os.path import join, isdir, realpath
-from os import mkdir
+from os import makedirs
 import logging, logging.handlers
 
 import xdg.BaseDirectory
 
 cache_dir = realpath(join(xdg.BaseDirectory.xdg_cache_home, "gwibber"))
 if not isdir(cache_dir):
-  mkdir(cache_dir)
+  makedirs(cache_dir)
 
 LOG_FILENAME = join(cache_dir, "gwibber.log")
 loghandler = logging.handlers.RotatingFileHandler(
