@@ -5,6 +5,10 @@ SegPhault (Ryan Paul) - 11/22/2008
 """
 
 import os, sys
+from os.path import join, isdir, realpath, exists
+from os import makedirs, remove
+import Image, log
+from gwibber.microblog import network
 from const import *
 
 # Try to import * from custom, install custom.py to include packaging 
@@ -14,6 +18,7 @@ try:
 except:
   pass
 
+log.logger.name = "Gwibber Dispatcher Resources"
 
 PROGRAM_NAME = "gwibber"
 UI_DIR_NAME = "ui"
@@ -43,6 +48,42 @@ def get_twitter_keys():
   # Distros should register their own keys and not rely on the defaults
   return TWITTER_OAUTH_KEY, TWITTER_OAUTH_SECRET
 
+def get_avatar_path(url):
+  avatar_cache_dir = realpath(join(CACHE_BASE_DIR, "gwibber", "avatars"))
+  if not isdir(avatar_cache_dir):
+    makedirs(avatar_cache_dir)
+  avatar_cache_image = join(avatar_cache_dir, url.replace("/",""))
+
+  if not exists(avatar_cache_image) or len(open(avatar_cache_image, "r").read()) < 1:
+    f = file(avatar_cache_image, "w")
+    data = network.Download(url)
+    f.write(data.get_string())
+    f.close()
+    img_resize(avatar_cache_image, 48)
+
+  if len(open(avatar_cache_image, "r").read()) > 0:
+    return avatar_cache_image
+  return None
+
+def del_avatar(avatar):
+  if exists(avatar):
+    try:
+      remove(avatar)
+    except:
+      log.logger.error("Failed to remove avatar from cache: %s", avatar)
+
+def img_resize(img_path, size):
+  try:
+    image = Image.open(img_path)
+    x, y = image.size
+    if x != size or y != size:
+      # need to upsample limited palette images before resizing
+      if image.mode == 'P': image = image.convert('RGBA')
+      image.resize((size, size), Image.ANTIALIAS).save(img_path)
+  except Exception, e:
+    from traceback import format_exc
+    log.logger.error("Image resizing failed:\n%s", format_exc())
+
 def get_desktop_file():
   p = os.path.join(LAUNCH_DIR, "gwibber.desktop")
   if os.path.exists(p): return p
