@@ -1,5 +1,11 @@
 import gtk, dbus, resources, os, mx.DateTime, webbrowser
 from microblog.util.const import *
+# Try to import * from custom, install custom.py to include packaging 
+# customizations like distro API keys, etc
+try:
+  from microblog.util.custom import *
+except:
+  pass
 
 import gettext
 from gettext import ngettext
